 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

//////////////////////////////////////////////////////////////////////
// BackgroundData.cpp: implementation of the CBackgroundData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Davves.h"
#include "BackgroundData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "hightime.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBackgroundData::CBackgroundData(CWnd* pParent, CFileList* pFileList)
{
	m_pFileList = pFileList;
	m_pParent = pParent;

	m_bContinue = TRUE;
	m_bIsRunning = FALSE;
	m_strSearchQuery = NULL;
	m_nQueryLength = 0;
	m_nLastSearchId = 0;
	m_arQuery = NULL;
	m_nCurrentQuery = 0;
	m_dirMatch = new BYTE[m_pFileList->m_network.m_listInfo.m_nOfflineDirectoriesCount+m_pFileList->m_network.m_listInfo.m_nOnlineDirectoriesCount+1];
	memset(m_dirMatch,0,m_pFileList->m_network.m_listInfo.m_nOfflineDirectoriesCount+m_pFileList->m_network.m_listInfo.m_nOnlineDirectoriesCount);
	m_nOffset = 0;
}

CBackgroundData::~CBackgroundData()
{
	if (m_arQuery)
	{
		for (UINT i=0;i<m_nQueryCount;i++)
		{
			delete[] m_arQuery[i];
		}
		delete[] m_arQuery;
		m_arQuery = NULL;
	}
	
	if (m_dirMatch)
	{
		delete[] m_dirMatch;
		m_dirMatch = NULL;
	}
}

UINT searchThread(LPVOID pParam)
{
	CBackgroundData* pData = (CBackgroundData *)pParam;		
	pData->m_bContinue = TRUE;
	pData->m_bIsRunning = TRUE;

	pData->DoSearch();

	pData->m_bIsRunning = FALSE;

	
	return 0;
}	

bool CBackgroundData::startThread()
{
	if (stopThread())
	{
		m_thread = AfxBeginThread(searchThread,this,THREAD_PRIORITY_NORMAL);
		return true;
	}
	else
	{
		return false;
	}
}


bool CBackgroundData::stopThread()
{
	if (m_bIsRunning)
	{
		m_bContinue = false;
		m_thread->m_bAutoDelete = FALSE;
		WaitForSingleObject(m_thread->m_hThread,INFINITE);
		delete m_thread;
		return true;
	}
	else
	{
		return true;
	}
}

bool CBackgroundData::findFiles()
{
	/* Search */
	CListFile currentFile;
	bool fileHit = false;
	bool hit = false;
	bool sameFilename = false;
	UINT nHits = 0;
	UINT currentId;
	for(currentId=m_nLastSearchId;(m_bContinue && currentId<m_pFileList->m_network.m_listInfo.m_nTotalFileCount);currentId++)
	{
		sameFilename = (currentFile.m_nFilenameId == m_pFileList->m_network.m_pFiles[currentId].m_nFilenameId);

		if (!sameFilename)
			fileHit = m_pFileList->searchFile(currentId);

		if (fileHit)
		{
			hit = true;
		}
		else
		{
			if (m_dirMatch[m_pFileList->getDirectoryId(currentId)] == 1)
			{
				hit = false;
			}
			else if (m_dirMatch[m_pFileList->getDirectoryId(currentId)] == 2)
			{
				hit = true;
			}
			else
			{
				hit = m_pFileList->searchDirectory(currentId);
				m_dirMatch[m_pFileList->getDirectoryId(currentId)] = hit + 1;
			}
		}
		
		if (hit)
		{
			m_listResult.AddTail(currentId);
			nHits++;
		}
		currentFile = m_pFileList->m_network.m_pFiles[currentId];
	}

	return true;
}

void CBackgroundData::ParseQuery()
{
	//CParseQuery parser(m_SearchInfo.GetSearchQuery());
	CParseQuery parser((LPSTR)m_SearchInfo.m_strQuery);
	
	if (m_arQuery)
	{
		for (UINT i=0;i<m_nQueryCount;i++)
		{
			delete[] m_arQuery[i];
		}
		delete[] m_arQuery;
		m_arQuery = NULL;
	}

	m_arQuery = parser.getArray();
	ASSERT(m_arQuery);
	m_nQueryCount = parser.getLengthOfArray();
}

void CBackgroundData::setListDoc(CDavvesDoc *pDoc)
{
	m_pListDoc = pDoc;
}

void CBackgroundData::setListView(CDavvesView *pView)
{
	m_pListView = pView;
}

void CBackgroundData::SetSearchInfo(CSearchInfo info)
{
	m_SearchInfo = info;

	// Do some preparing
	switch (m_SearchInfo.m_nSearchType)
	{
	case DAV_SEARCH_STRING:
		ParseQuery();	
	case DAV_SEARCH_DIR:
	case DAV_SEARCH_FILENAME:
		memset(m_dirMatch,0,m_pFileList->m_network.m_listInfo.m_nOfflineDirectoriesCount+m_pFileList->m_network.m_listInfo.m_nOnlineDirectoriesCount);
		m_nLastSearchId = 0;
		m_nOffset = 0;
		m_listResult.RemoveAll();
		break;
	case DAV_SEARCH_BROWSE:
		m_nOffset += info.m_nOffsetMove;
		break;
	}
}

//////////////////////////////////////////////////////////////////////////////
// Main search proc
//////////////////////////////////////////////////////////////////////////////

void CBackgroundData::DoSearch()
{
	CHighTime startTime;
	CHighTimeSpan elapsedTime;
	startTime = CHighTime::GetPresentTime();
	
	switch (m_SearchInfo.m_nSearchType)
	{
	case DAV_SEARCH_STRING:
		SearchString();
		break;
	case DAV_SEARCH_BROWSE:
		BrowseOffset();
		break;
	case DAV_SEARCH_DIR:
		BrowseDir();
	case DAV_SEARCH_FILENAME:
		BrowseFilename();
		break;
	}	

//	if (m_bContinue)
		UpdateResult();

	elapsedTime = CHighTime::GetPresentTime() - startTime;
	m_pListDoc->m_SearchInfo = m_SearchInfo;
	m_pListDoc->m_pSearchResult->SetSearchSpan(elapsedTime.GetMilliSeconds());
	m_pListDoc->m_pSearchResult->SetSearchQuery(m_strSearchQuery);

	if (m_bContinue)
		PostMessage(m_pParent->m_hWnd,IDU_SEARCH,IDU_SEARCH_COMPLETE,0);
	else
		PostMessage(m_pParent->m_hWnd,IDU_SEARCH,IDU_SEARCH_ABORT,0);
}

//////////////////////////////////////////////////////////////////////////////
// The four diffrent search ways
//////////////////////////////////////////////////////////////////////////////

void CBackgroundData::SearchString()
{
	m_nCurrentQuery = 0;
	
	// First find files with first query in file/dirname
	if (!feedNextToken() || !findFiles())
		return;
	
	// Filter files with wrong size
	FilterSize();

	// Filter files that do not contain remaining queries
	while (feedNextToken())
	{
		findFilesInResult();
	}
}

void CBackgroundData::BrowseOffset()
{
	// Not much to do
}

void CBackgroundData::BrowseDir()
{
	m_pFileList->GetFilesInDir(m_SearchInfo.m_nDirId, m_listResult);
}

void CBackgroundData::BrowseFilename()
{
	// Not done
}

bool CBackgroundData::feedNextToken()
{
	if (m_nCurrentQuery < m_nQueryCount)
	{
		m_nQueryLength = strlen((LPSTR)m_arQuery[m_nCurrentQuery]);
		m_strSearchQuery = m_arQuery[m_nCurrentQuery];
		m_pFileList->setSearchQuery(m_strSearchQuery, m_nQueryLength);	
		memset(m_dirMatch,0,m_pFileList->m_network.m_listInfo.m_nOfflineDirectoriesCount+m_pFileList->m_network.m_listInfo.m_nOnlineDirectoriesCount);
		m_nCurrentQuery++;
		return true;
	}
	
	return false;
}

bool CBackgroundData::findFilesInResult()
{
	/* Search in result*/
	CListFile currentFile;
	bool fileHit = false;
	bool hit = false;
	bool sameFilename = false;
	int nHits = 0;
	UINT nMaxHits = m_pListDoc->m_pSearchResult->GetTotalCount();
	UINT currentId;
	int nResults = m_listResult.GetCount();
	for(int idx=0;idx<nResults;idx++)
	{
		currentId = m_listResult.RemoveHead();
		fileHit = m_pFileList->searchFile(currentId);

		if (fileHit)
			hit = true;
		else
			hit = m_pFileList->searchDirectory(currentId);
		
		if (hit)
		{
			m_listResult.AddTail(currentId);
			nHits++;
		}

		currentFile = m_pFileList->m_network.m_pFiles[currentId];
	}
	
	return true;
}

void CBackgroundData::FilterSize()
{
	// make sure we have something to rebuild list for...
	if (!m_SearchInfo.m_nMinFileSize && !m_SearchInfo.m_nMaxFileSize)
		return;

	bool hit;
	UINT currentId;
	int nResults = m_listResult.GetCount();

	/* Search in result*/

	for(int idx=0;idx<nResults;idx++)
	{
		currentId = m_listResult.RemoveHead();
		
		hit = true;

		if (m_SearchInfo.m_nMinFileSize)
			hit &= (m_pFileList->getFilesize(currentId)>m_SearchInfo.m_nMinFileSize);

		if (m_SearchInfo.m_nMaxFileSize)
			hit &= (m_pFileList->getFilesize(currentId)<m_SearchInfo.m_nMaxFileSize);

		if (hit)
		{
			m_listResult.AddTail(currentId);
		}
	}
}

int CBackgroundData::getOffset()
{
	return m_nOffset;
}

void CBackgroundData::UpdateResult()
{
	if (!m_listResult.IsEmpty())
	{
		POSITION pos = m_listResult.FindIndex(m_nOffset);

		for (int i=0;(i<(m_listResult.GetCount()-m_nOffset) && i<m_pListDoc->m_pSearchResult->GetSize());i++)
		{
			int currentId = m_listResult.GetNext(pos);

			CDownloadFile* file = new CDownloadFile;
			file->m_nFileId = currentId;
			file->m_nFilesize = m_pFileList->getFilesize(currentId);
			file->m_nFileType = m_pFileList->GetFileTypeFromId(currentId);
			file->m_strFilename = m_pFileList->getFilename(currentId);
			file->m_nFilenameLength = m_pFileList->getFilenameLength(currentId);

			file->m_pShare = m_pFileList->GetShare(m_pFileList->GetShareId(currentId));
			
			file->m_strSharename = m_pFileList->getDirectorynameFromFileId(currentId);
			file->m_nSharenameLength = m_pFileList->getDirectorynameLengthFromFileId(currentId);

			m_pListDoc->m_pSearchResult->SetAt(i,file);
		}
	}

	m_pListDoc->m_pSearchResult->SetOffset(m_nOffset);
	m_pListDoc->m_pSearchResult->SetTotalCount(m_listResult.GetCount());
}

/////////////
// PARSE QUERY

CParseQuery::CParseQuery(LPCSTR strQuery)
{
	m_strQuery.Format("%s",strQuery);
	parse();
}

bool CParseQuery::parse()
{
	while (!m_strQuery.IsEmpty())
	{
		m_strQuery.TrimLeft("+ \t");

		int nIndex = m_strQuery.Find(" ");
	
		if (nIndex<0)
			nIndex = m_strQuery.GetLength();

		CString tmp = m_strQuery.Left(nIndex);
		m_listTokens.AddTail(tmp);
		m_strQuery.Delete(0,nIndex);
	}
	return true;
}

unsigned char** CParseQuery::getArray()
{
	unsigned char** arRes = new unsigned char*[m_listTokens.GetCount()];
	CString tmp;
	POSITION pos = m_listTokens.GetHeadPosition();

	for (int i=0;i<m_listTokens.GetCount();i++)
	{
		tmp = m_listTokens.GetNext(pos);
		arRes[i] = new unsigned char[tmp.GetLength()+1];
		memcpy(arRes[i],tmp.GetBuffer(0),tmp.GetLength());
		arRes[i][tmp.GetLength()] = '\0';
	}
	return arRes;
}

UINT CParseQuery::getLengthOfArray()
{
	return m_listTokens.GetCount();
}



