 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// BoyerMoore.cpp: implementation of the CBoyerMoore class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Davves.h"
#include "BoyerMoore.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////
// Boyer Moore Class
///////////////////////////////////////////////////////////////////////

unsigned char UPPERCHAR[256] = { 
		0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
		16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
		32,'!','"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
		'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?',
		'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
		'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
		'`','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
		'P','Q','R','S','T','U','V','W','X','Y','Z','{','|','}','~',127,
		128,129,130,131,142,133,143,128,136,137,138,139,140,141,142,143,
		144,145,146,147,153,149,150,151,152,153,154,155,156,157,158,159,
		160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
		176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,
		192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,
		208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,
		224,225,226,227,196,197,230,231,232,233,234,235,236,237,238,239,
		240,241,242,243,244,245,214,247,248,249,250,251,252,253,254,255,
		};

CBoyerMoore::CBoyerMoore()
{
}

bool CBoyerMoore::setQuery(unsigned char* strQuery, int nLength)
{
	m_strSearchQuery = strQuery;
	UpperCase();
	m_nLength = nLength;
	preBmGs((unsigned char *)m_strSearchQuery, m_nLength, m_bmGs);
	preBmBc((unsigned char *)m_strSearchQuery, m_nLength, m_bmBc);
	return true;
}

void CBoyerMoore::preBmBc(unsigned char *x, int m, unsigned char bmBc[]) {
	int i;
	
	for (i = 0; i < ASIZE; ++i)
		bmBc[i] = m;
	for (i = 0; i < m - 1; ++i)
		bmBc[UPPERCHAR[x[i]]] = m - i - 1;
}


void CBoyerMoore::suffixes(unsigned char *x, int m, unsigned char *suff) {
	int f, g, i;
	
	suff[m - 1] = m;
	g = m - 1;
	for (i = m - 2; i >= 0; --i) {
		if (i > g && suff[i + m - 1 - f] < i - g)
			suff[i] = suff[i + m - 1 - f];
		else {
			if (i < g)
				g = i;
			f = i;
			while (g >= 0 && UPPERCHAR[x[g]] == UPPERCHAR[x[g + m - 1 - f]])
				--g;
			suff[i] = f - g;
		}
	}
}

void CBoyerMoore::preBmGs(unsigned char *x, int m, unsigned char bmGs[]) 
{
	int i, j;
	unsigned char suff[XSIZE];
	
	suffixes(x, m, suff);
	
	for (i = 0; i < m; ++i)
		bmGs[i] = m;
	j = 0;
	for (i = m - 1; i >= -1; --i)
		if (i == -1 || suff[i] == i + 1)
			for (; j < m - 1 - i; ++j)
				if (bmGs[j] == m)
					bmGs[j] = m - 1 - i;
				for (i = 0; i <= m - 2; ++i)
					bmGs[m - 1 - suff[i]] = m - 1 - i;
}

bool CBoyerMoore::search(unsigned char *y, int n) 
{
	int i, j;
	
	j = 0;
	while (j <= n - m_nLength) 
	{
		for (i = m_nLength - 1; i >= 0 && m_strSearchQuery[i] == UPPERCHAR[y[i + j]]; --i);
		if (i < 0) {
			return true;
		}
		else
			j += __max(m_bmGs[i], m_bmBc[UPPERCHAR[y[i + j]]] - m_nLength + 1 + i);
	}
	return false;
}

void CBoyerMoore::UpperCase()
{
	int nLength = StrLength(m_strSearchQuery);
	for (int i=0;i<nLength;i++)
	{
		m_strSearchQuery[i] = UPPERCHAR[(BYTE)m_strSearchQuery[i]];
	}
}

int CBoyerMoore::StrLength(unsigned char* str)
{
	int i;
	for (i=0;str[i]!=0;i++);
	return i;
}
