 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// DavvesView.cpp : implementation of the CDavvesView class
//

#include "stdafx.h"
#include "Davves.h"

#include "DavvesDoc.h"
#include "DavvesView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDavvesView

IMPLEMENT_DYNCREATE(CDavvesView, CListView)

BEGIN_MESSAGE_MAP(CDavvesView, CListView)
	//{{AFX_MSG_MAP(CDavvesView)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_COMMAND(ID_LIST_FOCUS, OnListFocus)
	ON_NOTIFY_REFLECT(LVN_ODFINDITEM, OnOdfinditem)
	ON_NOTIFY(HDN_ITEMCLICKA, 0, OnHeaderClicked) 
	ON_NOTIFY(HDN_ITEMCLICKW, 0, OnHeaderClicked)
	ON_NOTIFY_REFLECT ( NM_CUSTOMDRAW, OnCustomdraw )
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDavvesView construction/destruction

CDavvesView::CDavvesView()
{
	m_font.CreatePointFont(85,"Tahoma");
	m_nSortedCol = DAV_VIEW_FILENAME; 
	m_nSortOrder = DAV_SORT_ORIG;
	m_strNoItems = NULL;
	CString strNoItems = "There are no items to show in this view";
	SetNoItemsText(strNoItems);
	m_pSelection = NULL;
}

CDavvesView::~CDavvesView()
{
	if (m_pSelection)
	{
		delete[] m_pSelection;
		m_pSelection = NULL;
	}

	if (m_strNoItems)
	{
		delete[] m_strNoItems;
		m_strNoItems = NULL;
	}
}

BOOL CDavvesView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_OWNERDATA;
	cs.style |= LVS_REPORT;
	return CListView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDavvesView drawing

void CDavvesView::OnInitialUpdate()
{
	CListView::OnInitialUpdate();

	CDavvesApp* app = (CDavvesApp *)AfxGetApp();

	//GetListCtrl().SetCallbackMask(LVIS_SELECTED); 

	int nFilenameWidth = (int)app->GetProfileInt("View","FilenameWidth",200);
	int nSizeWidth = (int)app->GetProfileInt("View","SizeWidth",100);
	int nShareWidth = (int)app->GetProfileInt("View","ShareWidth",200);
	int nPingWidth = (int)app->GetProfileInt("View","PingWidth",70);
	int nDistanceWidth = (int)app->GetProfileInt("View","DistanceWidth",70);
	int nAccessWidth = (int)app->GetProfileInt("View","AccessWidth",70);

	CListCtrl& list = GetListCtrl();
	list.SetFont(&m_font);
	list.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);

	list.DeleteAllItems();
	
	list.DeleteColumn(DAV_VIEW_FILENAME);
	list.DeleteColumn(DAV_VIEW_FILESIZE);
	list.DeleteColumn(DAV_VIEW_SHARE);
	list.DeleteColumn(DAV_VIEW_PING);
	list.DeleteColumn(DAV_VIEW_DISTANCE);
	list.DeleteColumn(DAV_VIEW_ACCESS);

	list.InsertColumn(DAV_VIEW_FILENAME,"Filename",LVCFMT_LEFT,nFilenameWidth);
	list.InsertColumn(DAV_VIEW_FILESIZE,"Size",LVCFMT_RIGHT,nSizeWidth);
	list.InsertColumn(DAV_VIEW_SHARE,"Path",LVCFMT_LEFT,nShareWidth);
	list.InsertColumn(DAV_VIEW_PING,"Ping",LVCFMT_LEFT,nPingWidth);
	list.InsertColumn(DAV_VIEW_DISTANCE,"Distance",LVCFMT_LEFT,nPingWidth);
	list.InsertColumn(DAV_VIEW_ACCESS,"Access",LVCFMT_LEFT,nAccessWidth);


	UpdateImageList();
}

/////////////////////////////////////////////////////////////////////////////
// CDavvesView diagnostics

#ifdef _DEBUG
void CDavvesView::AssertValid() const
{
	CListView::AssertValid();
}

void CDavvesView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CDavvesDoc* CDavvesView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDavvesDoc)));
	return (CDavvesDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDavvesView message handlers

void CDavvesView::OnDestroy() 
{
	CDavvesApp* app = (CDavvesApp *)AfxGetApp();
	CListCtrl& list = GetListCtrl();

	app->WriteProfileInt("View","FilenameWidth",list.GetColumnWidth(DAV_VIEW_FILENAME));
	app->WriteProfileInt("View","SizeWidth",list.GetColumnWidth(DAV_VIEW_FILESIZE));
	app->WriteProfileInt("View","ShareWidth",list.GetColumnWidth(DAV_VIEW_SHARE));
	app->WriteProfileInt("View","PingWidth",list.GetColumnWidth(DAV_VIEW_PING));
	app->WriteProfileInt("View","DistanceWidth",list.GetColumnWidth(DAV_VIEW_DISTANCE));
	app->WriteProfileInt("View","AccessWidth",list.GetColumnWidth(DAV_VIEW_ACCESS));
	
	CListView::OnDestroy();
}

void CDavvesView::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
	LVITEM* pItem= &(pDispInfo)->item;
	CDavvesDoc* pDoc = GetDocument();
	
	if (pItem->mask & LVIF_TEXT) //valid text buffer?
	{
		CDownloadFile* pFile = pDoc->m_pSearchResult->GetAt(pItem->iItem);	
		ASSERT(pFile);
		int nPing;
		switch(pItem->iSubItem)
		{
			case DAV_VIEW_FILENAME: 
				memcpy(pItem->pszText, pFile->m_strFilename,pFile->m_nFilenameLength);
				pItem->pszText[pFile->m_nFilenameLength] = '\0';
				break;
			case DAV_VIEW_FILESIZE: 
				sprintf(pItem->pszText,	"%d KB",pFile->m_nFilesize);
				break;
			case DAV_VIEW_SHARE: 
				memcpy(pItem->pszText, pFile->m_strSharename,pFile->m_nSharenameLength);
				pItem->pszText[pFile->m_nSharenameLength] = '\0';
				break;
			case DAV_VIEW_PING:
				nPing = pFile->m_pShare->m_nPing;
				if (nPing==-1)
					lstrcpy(pItem->pszText,"?");
				else if (nPing>=0)
					itoa(nPing,pItem->pszText,10);	
				else
					lstrcpy(pItem->pszText,"-");
				break;
			case DAV_VIEW_DISTANCE:
				itoa(pFile->m_pShare->m_nDistance,pItem->pszText,10);
				break;
			case DAV_VIEW_ACCESS:
				itoa(pFile->m_pShare->m_nAccessHistory >> 1,pItem->pszText,2);
				break;
		}
	}
	if (pItem->mask & LVIF_IMAGE)
	{
		CDownloadFile* pFile = pDoc->m_pSearchResult->GetAt(pItem->iItem);
		pItem->iImage =  pFile->m_nFileType;
	}
	/*
	if (pItem->mask & LVIF_STATE)
	{
		int nIsSelected = 0;
		TRACE("Request selection info for: %d, Statemask/mask/mask is %d/%d/\n",pItem->iItem,pItem->stateMask,pItem->state);
		if (m_mapSelected.Lookup(pItem->iItem,nIsSelected))
		{
			pItem->stateMask = LVIS_SELECTED;
			pItem->state = LVIS_SELECTED;
		}
	}
*/
	*pResult = 0;
}

BOOL CDavvesView::ResetContent()
{
	return GetListCtrl().DeleteAllItems();
}

void CDavvesView::OnHeaderClicked(NMHDR *pNMHDR, LRESULT *pResult)
{
	CDavvesDoc *pDoc = GetDocument();
    HD_NOTIFY *phdn = (HD_NOTIFY *) pNMHDR;

    if( phdn->iButton == 0 )
    {
		if (m_nSortedCol == phdn->iItem)
		{
			if (m_nSortedCol == 0)
				m_nSortOrder = (m_nSortOrder + 1) % DAV_SORT_ORDERS;
			else
				m_nSortOrder = (m_nSortOrder + 1) % 2;
		}
		else if (phdn->iItem != 0)
			m_nSortOrder = (m_nSortOrder+1) % 2;
			
		m_nSortedCol = phdn->iItem;
		pDoc->Sort(m_nSortedCol, m_nSortOrder);
    }
	Invalidate();
    *pResult = 0;
}

void CDavvesView::Update()
{
	CDavvesDoc *pDoc = GetDocument();
	pDoc->Sort(m_nSortedCol, m_nSortOrder);
	Invalidate();
}

void CDavvesView::OnCustomdraw(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( pNMHDR );
	CDavvesDoc *pDoc = GetDocument();
		
    // Take the default processing unless we set this to something else below.
    *pResult = CDRF_DODEFAULT;
	
    // First thing - check the draw stage. If it's the control's prepaint
    // stage, then tell Windows we want messages for every item.
	
    if ( CDDS_PREPAINT == pLVCD->nmcd.dwDrawStage )
	{
        *pResult = CDRF_NOTIFYITEMDRAW;
	}
    else if ( CDDS_ITEMPREPAINT == pLVCD->nmcd.dwDrawStage )
	{
       // Tell Windows to paint the control itself.
       *pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
	else if ( (CDDS_ITEMPREPAINT | CDDS_SUBITEM) == pLVCD->nmcd.dwDrawStage )
	{
        COLORREF crText, crBkgnd;
		CDownloadFile* pFile = pDoc->m_pSearchResult->GetAt(pLVCD->nmcd.dwItemSpec);

		// default colors
		crBkgnd = RGB(255,255,255);	
		int nPing = pFile->m_pShare->m_nPing;
		if (nPing != -2)
			crText = RGB(0,0,0);
		else
			crText = RGB(128,128,128); 

        if (pLVCD->iSubItem == m_nSortedCol)
        {
			crBkgnd = RGB(248,248,248);
		}
        
		if (pLVCD->iSubItem == 3)
		{
			if (nPing >= 0 && nPing < 10)
				crText = RGB(0,128,0);
			else if (nPing >= 10 && nPing < 80)
				crText = RGB(128,128,0);
			else if (nPing > 80 && nPing < 400)
				crText = RGB(128,0,0);
			else if (nPing == -1)
				crText = RGB(30,30,30);
			else
				crText = RGB(128,128,128);
		}
		
		// Store the colors back in the NMLVCUSTOMDRAW struct.
        pLVCD->clrText = crText;
        pLVCD->clrTextBk = crBkgnd;
		
        // Tell Windows to paint the control itself.
        *pResult = CDRF_DODEFAULT;
	}

}

void CDavvesView::OnPaint() 
{
	Default();
	
    if (GetListCtrl().GetItemCount() <= 0)
    {
        COLORREF clrText = RGB(80,80,80);
        COLORREF clrTextBk = RGB(255,255,255);

        CDC* pDC = GetDC();
        // Save dc state
        int nSavedDC = pDC->SaveDC();

        CRect rc;
        GetWindowRect(&rc);
        ScreenToClient(&rc);

        CHeaderCtrl* pHC;
        pHC = GetListCtrl().GetHeaderCtrl();
        if (pHC != NULL)
        {
            CRect rcH;
            pHC->GetItemRect(0, &rcH);
            rc.top += rcH.bottom + 2;
        }

        pDC->SetTextColor(clrText);
        pDC->SetBkColor(clrTextBk);
        pDC->FillRect(rc, &CBrush(clrTextBk));
        pDC->SelectObject(&m_font);
        rc.top += 10;
        pDC->DrawText((LPSTR)m_strNoItems, -1, rc, DT_CENTER | DT_WORDBREAK | DT_NOPREFIX | DT_NOCLIP);
		rc.top -= 10;

        // Restore dc
        pDC->RestoreDC(nSavedDC);
        ReleaseDC(pDC);
    }
	
	// Do not call CListView::OnPaint() for painting messages
	
}

void CDavvesView::SetNoItemsText(CString str)
{
	if (m_strNoItems)
	{
		delete[] m_strNoItems;
		m_strNoItems = NULL;
	}
	int nLength = str.GetLength();
	m_strNoItems = new unsigned char[nLength+1];
	memcpy(m_strNoItems,str.GetBuffer(nLength),nLength);
	m_strNoItems[nLength] = '\0';
}


void CDavvesView::UpdateImageList()
{
	CListCtrl& list = GetListCtrl();
	CMainFrame* pFrame = (CMainFrame *)GetParentFrame();
	list.SetImageList(pFrame->GetImageList(),LVSIL_SMALL);
}

void CDavvesView::OnListFocus() 
{
	GetListCtrl().SetFocus();
}

void CDavvesView::OnOdfinditem(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = -1;
	
	CDavvesDoc *pDoc = GetDocument();

	NMLVFINDITEM* pFindInfo = (NMLVFINDITEM*)pNMHDR;
	LVFINDINFO findInfo = pFindInfo->lvfi;
	if (findInfo.flags & (LVFI_PARTIAL | LVFI_STRING)) // have string
	{
		*pResult = pDoc->GetItemOfStr(findInfo.psz,pFindInfo->iStart);
	}
}

void CDavvesView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CListView::OnLButtonDblClk(nFlags, point);
}
