// DownloadControl.cpp: implementation of the CDownloadControl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DownloadControl.h"

const UINT    wm_Message = 
    RegisterWindowMessage(_T("B52BAD67-B2BD-48b0-B962-101ECF6B919F"));

const DWORD dwMemoryFileSize = 64 * 1024;

const LPCTSTR sMemoryFileName = _T("BF74C014-49B7-4564-A554-B83C1B2E136B");

const LPCTSTR sReadWriteLockName = _T("1D57E964-A516-48ba-9AB3-D01DB4D50EA4");

// maximum number of milliseconds to wait
const DWORD dwMaxWait = 200;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDownloadControl::CDownloadControl()
{
	m_bDLProcRunning = false;
	m_hFileMapping = NULL;
	m_pLock = NULL;
}

CDownloadControl::~CDownloadControl()
{

}

bool CDownloadControl::Init()
{
	// Create the Single Writer/Multiple Reader Guard
    m_pLock = new CIPCReadWriteLock(
        sReadWriteLockName,
        dwMaxWait);
    
	// Create file mapping which can contain dwMemoryFileSize characters
    m_hFileMapping = CreateFileMapping(
        INVALID_HANDLE_VALUE,           // system paging file
        NULL,                           // security attributes
        PAGE_READWRITE,                 // protection
        0,                              // high-order DWORD of size
        dwMemoryFileSize*sizeof(TCHAR), // low-order DWORD of size
        sMemoryFileName);               // name

    DWORD dwError = GetLastError();     // if ERROR_ALREADY_EXISTS 
							            // this instance is not first (other instance created file mapping)

    if (!m_hFileMapping)
    {
        return false;
    }
    else
    {
        m_pViewOfFile = (BYTE *)MapViewOfFile(
            m_hFileMapping,             // handle to file-mapping object
            FILE_MAP_ALL_ACCESS,        // desired access
            0,
            0,
            0);                         // map all file

        if (!m_pViewOfFile)
        {
			Close();
			return false;
        }

		m_pViewBeginning = m_pViewOfFile; // So we can rewind
	}
	return true;
}

bool CDownloadControl::Close()
{
	if ( m_hFileMapping )
    {
        if ( m_pViewOfFile )
        {
            UnmapViewOfFile(m_pViewOfFile);
        }

        CloseHandle(m_hFileMapping);
    }

    if ( m_pLock )
    {
        delete m_pLock;
    }	
	return true;
}

//////////////////////////////////////////////////////////////////////////////
// Publics procs
//////////////////////////////////////////////////////////////////////////////

void CDownloadControl::AddGroup(CDownloadGroup* group)
{
	m_lsGroups.AddTail(group);
}

void CDownloadControl::Download()
{
	// rewind 
	m_pViewOfFile = m_pViewBeginning;

    if ( m_pViewOfFile  &&  m_pLock )
    {
        // get write access to common memory block
        if ( m_pLock->WaitToWrite() )
        {
			// First byte; section count
			*m_pViewOfFile = (BYTE)m_lsGroups.GetCount();
			m_pViewOfFile++;
			
			POSITION pos = m_lsGroups.GetHeadPosition();
			while (pos)
			{
				CDownloadGroup* group = m_lsGroups.GetNext(pos);		
				
				group->PrepareData();
				
				// Block Type
				*m_pViewOfFile = DAV_IPC_BLOCK_GROUP;
				m_pViewOfFile++;
				
				// Block size
				*(WORD *)m_pViewOfFile = (WORD)group->GetDataSize();
				m_pViewOfFile+=2;
				
				// Block data
				memcpy(m_pViewOfFile,group->GetData(),group->GetDataSize());
				
				group->FreeData();
			}

			m_pLock->Done();
			UpdateDownload();
		}
	}

	// reset for new download
	m_pViewOfFile = m_pViewBeginning;
	m_lsGroups.RemoveAll();
}

void CDownloadControl::UpdateDownload()
{
	StartDownload();

	// Notify instance that it need to read from shared memory
    ::PostMessage(HWND_BROADCAST, 
                          wm_Message, 	
                         (WPARAM) 0,
                          0);    
}


//////////////////////////////////////////////////////////////////////////////
// Private procs
//////////////////////////////////////////////////////////////////////////////

void CDownloadControl::StartDownload()
{
	if (!m_bDLProcRunning)
	{
		STARTUPINFO startup;
		PROCESS_INFORMATION procinfo;
		
		::ZeroMemory(&startup, sizeof(startup));
		
		startup.cb = sizeof(startup);
		startup.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
		startup.wShowWindow = SW_SHOWNOACTIVATE;
		startup.hStdInput = NULL; // not used
		startup.hStdOutput = NULL;
		startup.hStdError = NULL;

		CString command = ".\\download.exe";
		
		LPTSTR cmd = command.GetBuffer(command.GetLength() + 1);
		
		BOOL starting = ::CreateProcess(
			NULL,				   // name of executable module
			cmd,				   // command line string
			NULL,					// SD
			NULL,					// SD
			TRUE,					// handle inheritance option
			0,						// creation flags
			NULL,					// new environment block
			NULL,					// current directory name
			&startup,				// startup information
			&procinfo				// process information
			);
		command.ReleaseBuffer();
		
		if (starting)
		{
			m_bDLProcRunning = TRUE;
			WaitForInputIdle(procinfo.hProcess,INFINITE);
			requestor.requestNotification(procinfo.hProcess, m_pParent);
		}		
	}
}
