 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// FileList.cpp: implementation of the CFileList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Davves.h"
#include "FileList.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileList::CFileList()
{
	m_bListLoaded = FALSE;
	m_strSearchQuery = NULL;
}

CFileList::~CFileList()
{
}

CNetwork::CNetwork()
{
	init();
}

void CNetwork::init()
{
	m_pMirrors = NULL;
	m_pFiles = NULL;
	m_pFilenames = NULL;
	m_pDirectoryNames = NULL;
	m_pShares = NULL;
	m_pIp = NULL;
	m_pFileTypes = NULL;

}

CNetwork::~CNetwork()
{
	deInit();
}

void CNetwork::deInit()
{
	if (m_pMirrors)
	{
		delete[] m_pMirrors;
		m_pMirrors = NULL;
	}
	if (m_pFiles)
	{
		delete[] m_pFiles;
		m_pFiles = NULL;
	}
	if (m_pFilenames)
	{
		delete[] m_pFilenames;
		m_pFilenames = NULL;
	}
	if (m_pDirectoryNames)
	{
		delete[] m_pDirectoryNames;
		m_pDirectoryNames = NULL;
	}

	// subblocks

	if (m_pShares)
	{
		for (UINT i=0;i<=m_listInfo.m_nMaxShareId;i++)
		{
			if (m_pShares[i])
				delete[] m_pShares[i];
			m_pShares[i] = NULL;
		}
		delete[] m_pShares;
		m_pShares = NULL;
	}

	if (m_pIp)
	{
		for (UINT i=0;i<=m_listInfo.m_nMaxShareId;i++)
		{
			if (m_pIp[i])
				delete[] m_pIp[i];
			m_pIp[i] = NULL;
		}
		delete[] m_pIp;
		m_pIp = NULL;
	}

	if (m_pFileTypes)
	{
		for (UINT i=0;i<m_nFileTypes;i++)
		{
			if (m_pFileTypes[i])
				delete[] m_pFileTypes[i];
			m_pFileTypes[i] = NULL;
		}
		delete[] m_pFileTypes;
		m_pFileTypes = NULL;
	}

	m_mapBigFilesizes.RemoveAll();

	POSITION pos = m_mapShares.GetStartPosition();
	while (pos)
	{
		CDownloadShare *share;
		WORD nID;
		m_mapShares.GetNextAssoc(pos,nID,share);
		m_mapShares.RemoveKey(nID);
		delete share;
	}
}

CIpAddr::CIpAddr()
{
	m_ip[0] = 0;
	m_ip[1] = 0;
	m_ip[2] = 0;
	m_ip[3] = 0;
}

CIpAddr::setData(BYTE inData[4])
{
	m_ip[0] = inData[0];
	m_ip[1] = inData[1];
	m_ip[2] = inData[2];
	m_ip[3] = inData[3];
}

CListMirror::CListMirror()
{
}
CListMirror::~CListMirror()
{
}

//////////////////////////////////////////////////////////////////////////////
// CDownloadFile
//////////////////////////////////////////////////////////////////////////////

CDownloadFile::CDownloadFile()
{
	m_nFileId = 0;
	m_pShare = NULL;
	m_nFilesize = 0;
	m_strFilename = NULL;
	m_strSharename = NULL;
	m_nFileType = 0;
}


CDownloadFile::~CDownloadFile()
{
}

//////////////////////////////////////////////////////////////////////////////
// CDownloadShare
//////////////////////////////////////////////////////////////////////////////

CDownloadShare::CDownloadShare()
{
	m_nID = 0;
	m_strShare = NULL;
	m_strComment = NULL;

	m_nShareLength = 0;
	m_nCommentLength = 0;

	m_nPing = -1;
	m_nDistance = -1;
	m_nIP1 = 0;
	m_nIP2 = 0;
	m_nAccessHistory = 0;

}


//////////////////////////////////////////////////////////////////////
// CListFile
//////////////////////////////////////////////////////////////////////

CListFile::CListFile()
{
	m_nFilenameId = 0;
	m_nDir = 0;
}


//////////////////////////////////////////////////////////////////////
// Datamaking for CDownloadFile
//////////////////////////////////////////////////////////////////////
	
void CDownloadFile::PrepareData()
{
	int i = 0;
	BYTE* pData;
	m_nDataSize = 0;

	// Find out size och data ,.... Just making sure
	m_nDataSize += 30; // For static info
	m_nDataSize += m_nFilenameLength + 1;
	m_nDataSize += m_nSharenameLength + 1;

	m_pData = new BYTE[m_nDataSize];
	pData = m_pData;
	
	// Write acc. spec
	// FileId
	*(int *)pData = m_nFileId;
	pData+=4;
	i+=4;

	// Filenamelength
	*pData = (BYTE)m_nFilenameLength + 1;
	pData++;
	i++;

	// Dirnameslength
	*pData = (BYTE)m_nSharenameLength + 1;
	pData++;
	i++;

	// Ping
	*(WORD *)pData = (WORD)m_pShare->m_nPing; //(WORD)m_nPing; // FIX ME!
	pData+=2;
	i+=2;

	// Distance
	*(WORD *)pData = (BYTE)m_pShare->m_nDistance;
	pData++;
	i++;

	// Access
	*pData = m_pShare->m_nAccessHistory;
	pData++;
	i++;

	// Filstorlek
	*(int *)pData = m_nFilesize;
	pData+=4;
	i+=4;

	// IP Adress
	CString tmp;
	tmp.Format("%d.%d.%d.%d",DG_IP_A,DG_IP_B,m_pShare->m_nIP1,m_pShare->m_nIP2);
	strcpy((char *)pData,tmp);
	pData+=16;
	i+=16;

	// Filename
	memcpy(pData,m_strFilename,m_nFilenameLength);
	pData+=m_nFilenameLength;
	*pData = '\0';
	pData++;
	i+=m_nFilenameLength+1;

	// Dirname
	memcpy(pData,m_strSharename,m_nSharenameLength);
	pData+=m_nSharenameLength;
	*pData = '\0';
	pData++;
	i+=m_nSharenameLength+1;

	ASSERT(i==m_nDataSize);

}

int CDownloadFile::GetDataSize()
{
	return m_nDataSize;	
}

BYTE* CDownloadFile::GetData()
{
	return m_pData;
}

void CDownloadFile::FreeData()
{
	delete[] m_pData;
}


//////////////////////////////////////////////////////////////////////
// Helper functions/classes
//////////////////////////////////////////////////////////////////////

BOOL CListInfo::parseData(UINT data[])
{
	m_nTotalFileCount = data[0];
	m_nTotalFilesize = data[1];
	m_nTotalUniqueFiles = data[2];
	m_nTotalUniqueSize = data[3];
	m_nTimeToleranceOffline = data[4];
	m_nTimeToleranceOnline = data[5];
	m_nOnlineFileCount = data[6];
	m_nOnlineFilesize = data[7];
	m_nOnlineDirectoriesCount = data[8];
	m_nOnlineSharesCount = data[9];
	m_nOfflineFileCount = data[10];
	m_nOfflineFilesize = data[11];
	m_nOfflineDirectoriesCount = data[12];
	m_nOfflineSharesCount = data[13];
    m_nMaxShareId = data[14];
	m_nTotalUniqueExactFiles = data[15];
	return true;
}

//////////////////////////////////////////////////////////////////////
// Load files
//////////////////////////////////////////////////////////////////////


bool CFileList::CheckList(CString strFilename)
{
	CFileStatus stat;
	if (CFile::GetStatus(strFilename,stat))
		return true;
	else
		return false;
}


bool CFileList::loadPerry(CString strFilename)
{
	UINT i; //for loops
	
	if (m_bListLoaded)
		return false;

	m_network.init();

	if (!m_listFile.Open(strFilename,CFile::modeRead | CFile::shareExclusive))
		return FALSE;

	m_hMap = CreateFileMapping((void *)m_listFile.m_hFile,NULL,PAGE_READONLY,0,0,"listfile");
	if (m_hMap != NULL && GetLastError() == ERROR_ALREADY_EXISTS) 
	{ 
		CloseHandle(m_hMap); 
		m_hMap = NULL; 
		return false;
	}
	
	m_hView = (BYTE *)MapViewOfFile(m_hMap, FILE_MAP_READ, 0, 0, 0); 

	/////////////////////////////////////////////////////////////
	//////////////
	// Header stuff
	////////////////////////////
	/////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////
	// Check (1) (2) and version (3)
	/////////////////////////////////////////////////////////////


	m_hView+=760; // skip 760 bytes of crap...
	
	if ((strncmp((char *)m_hView,"/MP3ListGen\\",12) != 0)) // check header
	{
		unLoadPerry();
		return false;
	}
	m_hView+=12;

	// check major version
	BYTE major = *m_hView;
	m_hView++;
	if (major != 3)
	{
		unLoadPerry();
		return false;
	}
	

	// skip minor version
	m_hView++;

	/////////////////////////////////////////////////////////////
	// Load: Mirrors
	/////////////////////////////////////////////////////////////
	m_network.m_nMirrors = *m_hView;
	m_hView++;
	if (m_network.m_pMirrors) delete[] m_network.m_pMirrors;
	m_network.m_pMirrors = new CListMirror[m_network.m_nMirrors];
	for (i=0;i<m_network.m_nMirrors;i++)
	{
		BYTE ip[3];

		m_network.m_pMirrors[i].m_nMirrorNameLength = *m_hView;
		m_hView++;

		m_network.m_pMirrors[i].m_strMirrorName = (char *)m_hView;
		m_hView+=m_network.m_pMirrors[i].m_nMirrorNameLength;
		
		memcpy(ip,m_hView,4);
		m_network.m_pMirrors[i].m_Ip;
		m_hView+=4;
	}

	/////////////////////////////////////////////////////////////
	// Load: List Info
	/////////////////////////////////////////////////////////////
	m_network.m_listInfo.m_strCreateDate = (char *)m_hView;
	m_hView+=16;

	UINT numericData[20];
	memcpy(numericData,m_hView,80);
	m_hView+=80;
	m_network.m_listInfo.parseData(numericData);

	/////////////////////////////////////////////////////////////
	// Load: Flags / Lazy me
	/////////////////////////////////////////////////////////////
	m_hView+=2; // flags 
	m_hView+=32; // ytterligare info

	/////////////////////////////////////////////////////////////
	// Load: Sub-blocks
	/////////////////////////////////////////////////////////////
	DWORD nBlockSize = *(DWORD *)m_hView;
	m_hView+=4;
	
	BYTE nBlocks = *m_hView;
	m_hView++;
		
	int numOfShares = m_network.m_listInfo.m_nOnlineSharesCount+m_network.m_listInfo.m_nOfflineSharesCount;

	for (i=0;i<nBlocks;i++)
	{
		BYTE nType = *m_hView;
		m_hView++;

		DWORD sizeOfBlock = *(DWORD *)m_hView;
		m_hView+=4;

		if (nType == 1)
		{
			int nDepth = *m_hView;
			m_hView++;

			int nSize = *m_hView;
			m_hView++;

			m_network.m_IPTree.SetDimensions(nSize,nDepth);
			BYTE* ipData = new BYTE[4+nDepth];

			for (int k=0;k<nSize;k++)
			{
				memcpy(ipData,m_hView,4+nDepth);
				m_network.m_IPTree.AddIP(ipData);
				m_hView+=4+nDepth;
			}
			delete[] ipData;
		}
		else if (nType == 2)
		{
			/*m_network.m_pShares = new unsigned char*[m_network.m_listInfo.m_nMaxShareId+1];
			memset(m_network.m_pShares,0,m_network.m_listInfo.m_nMaxShareId*4);
			for (int k=0;k<numOfShares;k++)
			{
				WORD nId = *(WORD *)m_hView;
				m_hView+=2;

				BYTE nLength = *m_hView;
				m_hView++;

				m_network.m_pShares[nId] = new unsigned char[nLength + 1];
				memcpy(m_network.m_pShares[nId],(const char *)m_hView,nLength);
				m_hView+=nLength;
				m_network.m_pShares[nId][nLength] = '\0';
				
			}*/
			m_network.m_mapShares.InitHashTable(m_network.m_listInfo.m_nMaxShareId);
			for (int k=0;k<numOfShares;k++)
			{
				CDownloadShare *share = new CDownloadShare;
				share->m_nID = *(WORD *)m_hView;
				m_hView+=2;

				share->m_nShareLength = *m_hView;
				m_hView++;
				
				share->m_strShare = (char *)m_hView;
				m_hView+=share->m_nShareLength;

				m_network.m_mapShares.SetAt(share->m_nID,share);
			}
		}
		else if (nType == 4) // IP addresses
		{
			/*m_network.m_pIp = new BYTE*[m_network.m_listInfo.m_nMaxShareId+1];
			memset(m_network.m_pIp,0,m_network.m_listInfo.m_nMaxShareId*4);
			for(int k=0;k<numOfShares;k++)
			{
				WORD nId = *(WORD *)m_hView;
				m_hView+=2;
				m_network.m_pIp[nId] = new BYTE[2];
				memcpy(m_network.m_pIp[nId],m_hView,2);
				m_hView+=2;
			}*/
			
			for(int k=0;k<numOfShares;k++)
			{
				WORD nId = *(WORD *)m_hView;
				m_hView+=2;
				
				CDownloadShare *share;
				if (m_network.m_mapShares.Lookup(nId, share))
				{
					share->m_nIP1 = *m_hView;
					m_hView++;
					share->m_nIP2 = *m_hView;
					m_hView++;

					share->m_nDistance = GetDistance(share);
				}				
			}
		}
		else if (nType == 5)
		{
			for(int k=0;k<numOfShares;k++)
			{
				WORD nId = *(WORD *)m_hView;
				m_hView+=2;
				
				CDownloadShare *share;
				if (m_network.m_mapShares.Lookup(nId, share))
				{
					share->m_nAccessHistory = *m_hView;
					m_hView++;

					share->m_nDistance = GetDistance(share);
				}				
			}
		}
		else if (nType == 9) // Big Filesizes
		{
			m_network.m_nBigFilesizes = *(DWORD *)m_hView;
			m_network.m_mapBigFilesizes.InitHashTable(10009); // This figure was calculated by a very advanced algorithm
			m_hView+=4;
			for(UINT k=0;k<m_network.m_nBigFilesizes;k++)
			{
				DWORD nId = *(DWORD *)m_hView;
				m_hView+=4;
				
				DWORD nFilesize = *(DWORD *)m_hView;			
				m_hView+=4;

				m_network.m_mapBigFilesizes.SetAt(nId, nFilesize);
			}
		}
		else if (nType == 10) // Filetypes
		{
			CString tmp;
			m_network.m_nFileTypes = *m_hView;
			m_hView++;
			m_network.m_pFileTypes = new unsigned char*[m_network.m_nFileTypes];
			for (UINT k=0;k<m_network.m_nFileTypes;k++)
			{
				memcpy(tmp.GetBuffer(15),m_hView,15);
				tmp.TrimRight();

				m_network.m_pFileTypes[k] = new unsigned char[tmp.GetLength()+2];
				m_network.m_pFileTypes[k][0] = '.';
				memcpy(m_network.m_pFileTypes[k]+1,tmp,tmp.GetLength());
				m_network.m_pFileTypes[k][tmp.GetLength()+1] = '\0';
				m_hView+=15;
			}
		}
		else
		{
			m_hView+=sizeOfBlock;
		}
	}

	/////////////////////////////////////////////////////////////
	//////////////
	// Load: Files and catalogs
	////////////////////////////
	/////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////
	// Check tag
	/////////////////////////////////////////////////////////////
	if (strncmp((char *)m_hView,"/MP3ListGen/MP3ListG",20))
	{
		return false;
	}
	m_hView+=20;
	
	/////////////////////////////////////////////////////////////
	// Load: Filesizes
	/////////////////////////////////////////////////////////////
	m_network.m_pFilesizes = (WORD *)m_hView; 
	m_hView+=m_network.m_listInfo.m_nTotalFileCount*2;

	m_hView+=8;

	/////////////////////////////////////////////////////////////
	// Load: DirID to ShareID
	/////////////////////////////////////////////////////////////
	m_network.m_pDir2ShareId = (unsigned short *)m_hView;
	m_hView+=2*(m_network.m_listInfo.m_nOnlineDirectoriesCount+m_network.m_listInfo.m_nOfflineDirectoriesCount);

	m_hView+=8;

	/////////////////////////////////////////////////////////////
	// Load: FileId to FilenameID and DirID
	/////////////////////////////////////////////////////////////
	m_network.m_pFiles = new CListFile[m_network.m_listInfo.m_nTotalFileCount];
	memcpy(m_network.m_pFiles,m_hView,m_network.m_listInfo.m_nTotalFileCount*8);
	m_hView+=m_network.m_listInfo.m_nTotalFileCount*8;

	m_hView+=8;

	/////////////////////////////////////////////////////////////
	// Load: Directory name lengths
	/////////////////////////////////////////////////////////////
	m_network.m_pDirectoriesLength = m_hView;
	m_hView+=m_network.m_listInfo.m_nOnlineDirectoriesCount+m_network.m_listInfo.m_nOfflineDirectoriesCount;

	m_hView+=8;

	/////////////////////////////////////////////////////////////
	// Load: Filenames lengths
	/////////////////////////////////////////////////////////////
	m_network.m_pFilenamesLength = m_hView;
	m_hView+=m_network.m_listInfo.m_nTotalUniqueExactFiles;

	m_hView+=8;
	
	/////////////////////////////////////////////////////////////
	// Load: Offsets to directories
	/////////////////////////////////////////////////////////////
	m_network.m_lSizeOfDirectories = 0;
	m_network.m_pDirectoryNames = new unsigned char*[m_network.m_listInfo.m_nOnlineDirectoriesCount+m_network.m_listInfo.m_nOfflineDirectoriesCount];
	for (i=0;i<(m_network.m_listInfo.m_nOnlineDirectoriesCount+m_network.m_listInfo.m_nOfflineDirectoriesCount);i++)
	{
		m_network.m_pDirectoryNames[i] = (unsigned char*)(m_hView + m_network.m_lSizeOfDirectories);
		m_network.m_lSizeOfDirectories += m_network.m_pDirectoriesLength[i];
	}

	m_hView+=m_network.m_lSizeOfDirectories;
	m_hView+=8;

	/////////////////////////////////////////////////////////////
	// Load: Offsets to filenames
	/////////////////////////////////////////////////////////////

	m_network.m_lSizeOfFilenames = 0;
	m_network.m_pFilenames = new unsigned char*[m_network.m_listInfo.m_nTotalFileCount];
	
	UINT nFilenameId = 1;
	for (UINT nFileId=0;nFileId<m_network.m_listInfo.m_nTotalFileCount;nFileId++)
	{
		if (!(m_network.m_pFiles[nFileId].m_nFilenameId == nFilenameId))
		{
			m_network.m_lSizeOfFilenames += m_network.m_pFilenamesLength[nFilenameId-1];
			nFilenameId++;
		}
		m_network.m_pFilenames[nFileId] = (unsigned char*)(m_hView + m_network.m_lSizeOfFilenames);
	}


	m_hView+=m_network.m_lSizeOfFilenames;
	m_hView+=8;

	m_bListLoaded = TRUE;
	return true;
}

bool CFileList::unLoadPerry()
{
	if (m_bListLoaded)
		m_network.deInit();

	m_bListLoaded = FALSE;
	UnmapViewOfFile(m_hView);
	CloseHandle(m_hMap);
	m_listFile.Close();
	return true;
}

bool CFileList::searchFile(UINT nFileId)
{
	return m_boyerMoore.search((unsigned char *)m_network.m_pFilenames[nFileId], // source
		m_network.m_pFilenamesLength[m_network.m_pFiles[nFileId].m_nFilenameId-1]);
}

bool CFileList::searchDirectory(UINT nFileId)
{
	return m_boyerMoore.search((unsigned char *)m_network.m_pDirectoryNames[m_network.m_pFiles[nFileId].m_nDir-1],
		m_network.m_pDirectoriesLength[m_network.m_pFiles[nFileId].m_nDir-1]);
}

UINT CFileList::getFilesize(UINT nFileId)
{
	UINT nFilesize;

	if (m_network.m_pFilesizes[nFileId] == 65535)
		m_network.m_mapBigFilesizes.Lookup(nFileId+1,nFilesize);
	else
		nFilesize = (UINT)m_network.m_pFilesizes[nFileId];
	return nFilesize;
}


UINT CFileList::getCreated(LPSTR strCreatedDate)
{
	memcpy(strCreatedDate,m_network.m_listInfo.m_strCreateDate,16);
	strCreatedDate[16] = '\0';
	return 1;
}

LPSTR CFileList::getIPOfFile(UINT nFileId)
{
	/*
	LPSTR ret = new char[16];
	int nLength = sprintf(ret,"130.236.%d.%d",
		m_network.m_pIp[m_network.m_pDir2ShareId[m_network.m_pFiles[nFileId].m_nDir - 1]][0],
		m_network.m_pIp[m_network.m_pDir2ShareId[m_network.m_pFiles[nFileId].m_nDir - 1]][1]);
	ret[nLength] = '\0';*/
	return NULL;
}

int CFileList::GetDistance(CDownloadShare *share)
{
	BYTE ip1[] =	{	130,
					236,
					226,
					13
				};

	
	BYTE ip2[] =	{	130,
					236,
					share->m_nIP1,
					share->m_nIP2,
				};

	return m_network.m_IPTree.CalcDistance(ip1,ip2);
}

bool CFileList::setSearchQuery(unsigned char* strQuery, UINT nLength)
{
	m_strSearchQuery = strQuery;
	m_nLength = nLength;
	m_boyerMoore.setQuery(m_strSearchQuery,nLength);
	return true;
}


UINT CFileList::getDirectoryName(CString &strDir, UINT nFileId)
{
	if (memcpy(strDir.GetBuffer(m_network.m_pDirectoriesLength[m_network.m_pFiles[nFileId].m_nDir-1]),
		m_network.m_pDirectoryNames[m_network.m_pFiles[nFileId].m_nDir-1],
		m_network.m_pDirectoriesLength[m_network.m_pFiles[nFileId].m_nDir-1]) == NULL)
	{
		return 0;
	}
	else
	{
		return 1;
	}	
}

UINT CFileList::getFilename(CString& strFilename, UINT nFileId)
{
	if (memcpy(strFilename.GetBuffer(m_network.m_pFilenamesLength[m_network.m_pFiles[nFileId].m_nFilenameId-1]),
		m_network.m_pFilenames[nFileId],
		m_network.m_pFilenamesLength[m_network.m_pFiles[nFileId].m_nFilenameId-1]) == NULL)
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

UINT CFileList::GetShareId(UINT nFileId)
{
	return m_network.m_pDir2ShareId[getDirectoryId(nFileId)];
}

CDownloadShare* CFileList::GetShare(UINT nShareID)
{
	CDownloadShare* share;
	if (m_network.m_mapShares.Lookup(nShareID,share))
		return share;
	else
		return NULL;
}

UINT CFileList::GetFileTypeCount()
{
	return m_network.m_nFileTypes;
}

unsigned char* CFileList::GetFileType(UINT nFileType)
{
	ASSERT (nFileType<m_network.m_nFileTypes);
	return m_network.m_pFileTypes[nFileType];
}

UINT CFileList::GetFileTypeFromId(UINT nFileId)
{
	unsigned char* strFilename = getFilename(nFileId);
	UINT nLength = getFilenameLength(nFileId) - 1;
	
	for (UINT fileType=0;fileType<m_network.m_nFileTypes;fileType++)
	{
		unsigned char* strFileType = GetFileType(fileType);
		UINT nTypeLength = strlen((LPSTR)strFileType) - 1;
		for (UINT i=0;(i<nTypeLength) && (strFileType[nTypeLength-i] == strFilename[nLength-i]);i++);

		if (i==nTypeLength)
			return fileType;
	}
	return 0;
}

void CFileList::GetInfo(CListInfo &info)
{
	info = m_network.m_listInfo;
}

void CFileList::GetFilesInDir(UINT nDirId, CIntList& list)
{
	UINT nRealId = nDirId + 1;
	for (UINT i=0;i<m_network.m_listInfo.m_nTotalFileCount;i++)
	{
		if (m_network.m_pFiles[i].m_nDir == nRealId)
			list.AddTail(i);
	}
}
