#if !defined(AFX_FILELIST_H__F223F018_890F_41B7_BC43_2A75E3906682__INCLUDED_)
#define AFX_FILELIST_H__F223F018_890F_41B7_BC43_2A75E3906682__INCLUDED_

 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>
#include "IPTree.h"
#include "BoyerMoore.h"

#define DAV_FILE_FILENAME 0
#define DAV_FILE_FILESIZE 1
#define DAV_FILE_SHARENAME 2
#define DAV_FILE_PING 3
#define DAV_FILE_DISTANCE 4
#define DAV_FILE_ACCESS 5

typedef CList<UINT,UINT> CIntList;

class CIpAddr
{
private:
	UINT m_ip[4];
public:
	CIpAddr();
	setData(BYTE inData[4]);
};

class CListInfo
{
public:
	BOOL parseData(UINT data[20]);
	UINT m_nTotalFilesize;
	UINT m_nTotalFileCount;
	UINT m_nTotalUniqueFiles;
	UINT m_nTotalUniqueSize;
	UINT m_nTimeToleranceOffline;
	UINT m_nTimeToleranceOnline;
	UINT m_nOnlineFileCount;
	UINT m_nOnlineFilesize;
	UINT m_nOnlineDirectoriesCount;
	UINT m_nOnlineSharesCount;
	UINT m_nOfflineFileCount;
	UINT m_nOfflineFilesize;
	UINT m_nOfflineDirectoriesCount;
	UINT m_nOfflineSharesCount;
    UINT m_nMaxShareId;
	UINT m_nTotalUniqueExactFiles;
	LPSTR m_strCreateDate;
};

class CListMirror  
{
public:
	UINT m_nMirrorNameLength;
	LPSTR m_strMirrorName;
	CIpAddr m_Ip;
	CListMirror();
	~CListMirror();
};

class CDownloadShare
{
public:
	CDownloadShare();
	
	int m_nID;
	LPSTR m_strShare;
	LPSTR m_strComment;

	int m_nShareLength;
	int m_nCommentLength;

	int m_nPing;
	int m_nDistance;
	int m_nIP1;
	int m_nIP2;
	BYTE m_nAccessHistory;
};

typedef CTypedPtrMap<CMapWordToPtr, WORD, CDownloadShare*> CShareMap;


class CDownloadFile
{
private:
	BYTE* m_pData;
	int m_nDataSize;
public:
	CDownloadFile();
	unsigned char* m_strFilename;
	unsigned char* m_strSharename;	

	int m_nFilenameLength;
	int m_nSharenameLength;
	
	int m_nFileId;		
	int m_nFilesize;
	CDownloadShare *m_pShare;
	int m_nFileType;
	LPSTR m_strIp;
	~CDownloadFile();

	void PrepareData();
	void FreeData();
	BYTE* GetData();
	int GetDataSize();
};

class CListFile
{
public:
	UINT m_nFilenameId;
	UINT m_nDir;
	CListFile();
};

class CNetwork  
{
public:
	void deInit();
	void init();
	CNetwork();
	virtual ~CNetwork();

	CListFile* m_pFiles;
	CListInfo m_listInfo;

	long m_lSizeOfDirectories;
	long m_lSizeOfFilenames;
	
	BYTE m_nMirrors;
	UINT m_nCurrentFilenameIdSum;
	UINT m_nCurrentFilenameId;
	
	CListMirror* m_pMirrors;
	WORD* m_pFilesizes;
	
	CMap<UINT, UINT, UINT, UINT> m_mapBigFilesizes;
	
	UINT m_nBigFilesizes;
	WORD* m_pDir2ShareId;

	BYTE* m_pDirectoriesLength;
	BYTE* m_pFilenamesLength;

	UINT* m_pDirectoriesIndex;
	UINT* m_pFilenamesIndex;

	unsigned char** m_pDirectoryNames;
	unsigned char** m_pFilenames;

	unsigned char** m_pShares;
	BYTE** m_pIp;

	unsigned char** m_pFileTypes;
	UINT m_nFileTypes;

	CIPTree m_IPTree;
	CShareMap m_mapShares;
};

class CFileList  
{
public:
	void GetFilesInDir(UINT nDirId, CIntList& list);
	bool CheckList(CString strFilename);
	void GetInfo(CListInfo& info);
	UINT GetFileTypeFromId(UINT nFileId);
	unsigned char* GetFileType(UINT nFileType);
	UINT GetFileTypeCount();
	UINT GetShareId(UINT nFileId);
	CDownloadShare* GetShare(UINT nShareID);
	UINT getFilename(CString& strFilename, UINT nFileId);
	UINT getDirectoryName(CString& strDir, UINT nFileId);
	bool setSearchQuery(unsigned char* strQuery, UINT nLength);
	LPSTR getIPOfFile(UINT nFileId);
	int GetDistance(CDownloadShare *share);

	__inline unsigned char* getFilename(UINT nFileId) { return m_network.m_pFilenames[nFileId]; }
	__inline unsigned char* getDirectorynameFromFileId(UINT nFileId) { return m_network.m_pDirectoryNames[m_network.m_pFiles[nFileId].m_nDir - 1]; }
	__inline UINT getFilenameLength(UINT nFileId) {	return m_network.m_pFilenamesLength[m_network.m_pFiles[nFileId].m_nFilenameId - 1]; }
	__inline UINT getDirectorynameLengthFromFileId(UINT nFileId) { return m_network.m_pDirectoriesLength[m_network.m_pFiles[nFileId].m_nDir - 1]; }
	__inline UINT getDirectoryId(UINT nFileId) { return m_network.m_pFiles[nFileId].m_nDir - 1; }
	UINT getCreated(LPSTR strCreatedDate);
	UINT getFilesize(UINT nFileId);
	bool searchFile(UINT nFileId);
	bool searchDirectory(UINT nFileId);
	bool unLoadPerry();
	bool loadPerry(CString strFilename);
	CFileList();
	virtual ~CFileList();
	CNetwork m_network;
private:
	UINT m_nLength;
	unsigned char* m_strSearchQuery;
	UINT m_nTypeOfSearch;
	CBoyerMoore m_boyerMoore;

	BOOL m_bListLoaded;
	HANDLE m_hMap;
	BYTE* m_hView;
	CFile m_listFile;
};

#endif // !defined(AFX_FILELIST_H__F223F018_890F_41B7_BC43_2A75E3906682__INCLUDED_)
