// IPCReadWriteLock.h: interface for the CIPCReadWriteLock class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IPCREADWRITELOCK_H__D9F75169_2396_49C0_BB4F_91FBC985C2B6__INCLUDED_)
#define AFX_IPCREADWRITELOCK_H__D9F75169_2396_49C0_BB4F_91FBC985C2B6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// The Single Writer/Multiple Reader Guard
// for inter-process communication.
// Written by: Alex Farber   alexm@cmt.co.il  
// 2001
//
// Based on CSWMRG class 
// Copyright (c) 2000 Jeffrey Richter
// From the book:
// Programming Applications for Microsoft Windows
// PUBLISHED BY
// Microsoft Press


// structure with counters kept in memory-mapped file
struct RW_COUNTERS
{
   int    m_nWaitingReaders; // Number of readers waiting for access
   int    m_nWaitingWriters; // Number of writers waiting for access
   int    m_nActive;         // Number of threads currently with access
};


class CIPCReadWriteLock  
{
public:
	CIPCReadWriteLock(LPCTSTR sName, DWORD dwMilliseconds);
	virtual ~CIPCReadWriteLock();

    BOOL WaitToRead();        // Call this to gain shared read access
    BOOL WaitToWrite();       // Call this to gain exclusive write access
    BOOL Done();              // Call this when done accessing the resource

protected:
    BOOL InitSyncObjects();

    BOOL m_bInitSucceded;       // TRUE if initialization succeded

    DWORD m_dwMilliseconds;
    TCHAR* m_sMutexName;
    TCHAR* m_sSemReadersName;
    TCHAR* m_sSemWritersName;
    TCHAR* m_sMemFileName;

    HANDLE m_hMutex;          // Permits exclusive access to other members
    HANDLE m_hsemReaders;     // Readers wait on this if a writer has access
    HANDLE m_hsemWriters;     // Writers wait on this if a reader has access

    // memory-mapped file used to keep counters:
    HANDLE m_hFileMapping;
    LPVOID m_pViewOfFile;

    int*    m_pnWaitingReaders; // Number of readers waiting for access
    int*    m_pnWaitingWriters; // Number of writers waiting for access
    int*    m_pnActive;         // Number of threads currently with access

};

/*
NOTES. This class should be used for inter-process synchronization.
For in-process syncronization it is better to use Jeffrey Richter's
CSWMRG class.

If number of instances of this class are used in the same process
with the same name (also NULL), they will protect the same common
resource.

Constructor if this class has two parameters:
1) sName - defines name of common resource to protect.
           May be any string.
2) dwMilliseconds - maximum number of milliseconds to wait.
           INFINITE value means waiting time-out interval
           never elapses.

  If dwMilliseconds = INFINITE and one process hangs while 
  reading/writing commong resource, all other processes trying
  to get access also hang. It is recommended to use some appropriate
  value for dwMilliseconds parameter which depends on the time 
  process reads/writes common resource.

  Reading/writing common resource should be as fast as possible
and should not include handling of Windows messages, waiting for
network requests etc.

*/



#endif // !defined(AFX_IPCREADWRITELOCK_H__D9F75169_2396_49C0_BB4F_91FBC985C2B6__INCLUDED_)
