// IPTree.cpp: implementation of the CIPTree class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "davves.h"
#include "IPTree.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIPTree::CIPTree()
{
	m_pTree = NULL;
	m_nDepth = 0;
	m_nSize = 0;
	m_nNextID = 0;
}

CIPTree::~CIPTree()
{
	if (m_pTree)
	{
		for (int i=0;i<m_nSize;i++)
		{
			delete[] m_pTree[i];
		}
		delete[] m_pTree;
		m_pTree = NULL;
	}
}

void CIPTree::SetDimensions(BYTE nSize, BYTE nDepth)
{
	// Can be done just one time!
	ASSERT(m_pTree == NULL);
	
	m_nSize = nSize;
	m_nDepth = nDepth;

	// Allocate data
	m_pTree = new BYTE*[m_nSize];
	for (int i=0;i<m_nSize;i++)
	{
		m_pTree[i] = new BYTE[4 + m_nDepth];
	}
}

void CIPTree::AddIP(BYTE treeItem[])
{
	memcpy(m_pTree[m_nNextID],treeItem,4+m_nDepth);
	m_nNextID++;
}

BYTE* CIPTree::GetTreePos(BYTE* ip)
{
	for (int i=0;i<m_nSize;i++)
	{
		if (ip[0] == m_pTree[i][0] &&
			ip[1] == m_pTree[i][1] &&
			ip[2] == m_pTree[i][2])
//			ip[3] == m_pTree[i][3])
		{
			return m_pTree[i]+4;
		}
	}
	return NULL;
}

int CIPTree::CalcDistance(BYTE ip1[], BYTE ip2[])
{
	BYTE* pPos1 = GetTreePos(ip1);
	BYTE* pPos2 = GetTreePos(ip2);

	if (pPos1 && pPos2)
		return m_nDepth - GetDiff(pPos1, pPos2, 0);
	else
		return -1;
}

int CIPTree::GetDiff(BYTE* nPos1, BYTE* nPos2, BYTE nDepth)
{
	if (nDepth < m_nDepth && *nPos1 == *nPos2)
		return GetDiff(nPos1+1, nPos2+1, nDepth+1);
	else
		return nDepth;
}
