 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Davves.h"
#include "MainFrm.h"
#include "DavvesDoc.h"
#include "DavvesView.h"
#include "TextDlg.h"
#include "InfoZip.h"
#include "ping.h"
#include "SAPrefsDialog.h"
#include "OptionsPaths.h"
#include "OptionsUI.h"
#include "OptionsDownload.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define DAV_HELP_BUFFER_SIZE 16384

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_BN_CLICKED(IDC_SEARCH_BUTTON, OnSearchButton)
	ON_WM_CLOSE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_ITEM_ADDTOQUEUE, OnItemAddtoqueue)
	ON_COMMAND(ID_ITEM_OPENDIRECTORY, OnItemOpendirectory)
	ON_COMMAND(ID_ITEM_EXECUTE, OnItemExecute)
	ON_COMMAND(ID_ITEM_PING, OnItemPing)
	ON_COMMAND(ID_LIST_UPDATE, OnListUpdate)
	ON_UPDATE_COMMAND_UI(ID_NEXT, OnUpdateNextButton)
	ON_UPDATE_COMMAND_UI(ID_PREV, OnUpdateBackButton)
	ON_COMMAND(ID_ITEM_ADDASSEPARATEGROUPS, OnItemAddasseparategroups)
	ON_COMMAND(ID_HELP_LICENCE, OnHelpLicence)
	ON_WM_SYSCOMMAND()
	ON_COMMAND(ID_VIEW_DOWNLOAD, OnViewDownload)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_FILE_OPTIONS, OnFileOptions)
	ON_COMMAND(ID_ITEM_LISTDIRECTORY, OnItemListdirectory)
	ON_COMMAND(ID_UP, OnUp)
	ON_UPDATE_COMMAND_UI(ID_UP, OnUpdateUp)
	ON_COMMAND(ID_DOWN, OnDown)
	ON_UPDATE_COMMAND_UI(ID_DOWN, OnUpdateDown)
	ON_COMMAND(ID_TOOLS_CHECKUPDATES, OnToolsCheckupdates)
	ON_COMMAND(ID_ITEM_COPYPATH, OnItemCopypath)
	ON_COMMAND(ID_ENTER, OnSearchButton)
	ON_COMMAND(ID_SEARCH_QUERY, OnSearchQueryButton)
	ON_MESSAGE(IDU_SEARCH, OnSearchCommand)
	ON_COMMAND(ID_PREV, OnBackButton)
	ON_COMMAND(ID_NEXT, OnNextButton)
	ON_UPDATE_COMMAND_UI(IDS_LIST_CREATED,OnUpdateListCreatedIndicator)
	ON_UPDATE_COMMAND_UI(IDS_LIST_DATA,OnUpdateListDataIndicator)
	ON_COMMAND(ID_HELP_CHECKVERSION, OnHelpCheckversion)
	//}}AFX_MSG_MAP
	// Global help commands
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP_WEB,OnWebHelp)
	ON_REGISTERED_MESSAGE(CWaitInfo::UWM_PROCESS_TERMINATED, OnCompletion)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR, // status line indicator
	IDS_LIST_CREATED,
	IDS_LIST_DATA
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pBackgroundData = NULL;
	m_bHaveList = false;
	m_bNext = false;
	m_bPrev = false;
	m_bDown = false;
	m_bUp = false;
	m_strNetworkFilename = "SUBnets MP3Lista - MP3Get.txt";
}

CMainFrame::~CMainFrame()
{
	if (m_pBackgroundData)
	{
		delete m_pBackgroundData;
		m_pBackgroundData = NULL;
	}
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	m_fontVerdana.CreatePointFont(85,"Tahoma");
		
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle());

	if (!m_wndDlgBar.Create(this, IDR_MAINFRAME, 
		CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create dialogbar\n");
		return -1;		// fail to create
	}

	if (!m_wndOptBar.Create(this, IDD_OPTIONS_BAR, 
		CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create options bar\n");
		return -1;		// fail to create
	}

	if (!m_wndReBar.Create(this, RBS_AUTOSIZE|RBS_DBLCLKTOGGLE|RBS_VARHEIGHT ) ||
		!m_wndReBar.AddBar(&m_wndToolBar) ||
		!m_wndReBar.AddBar(&m_wndDlgBar) ||
		!m_wndReBar.AddBar(&m_wndOptBar))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.GetToolBarCtrl().EnableToolTips(TRUE);
	m_wndToolBar.EnableToolTips(TRUE);
	m_wndReBar.EnableToolTips(TRUE);

	m_wndToolBar.SetFont(&m_fontVerdana);
	m_wndStatusBar.SetFont(&m_fontVerdana);
	m_wndDlgBar.SetFont(&m_fontVerdana);
	SetFont(&m_fontVerdana);

	m_Download.m_pParent = this;

	// Check for list
	bool bTry = true;
	bool bSuccess = false;
	if (!m_fileList.CheckList(m_strNetworkFilename))
	{
		MessageBox("Could not find list! Program will start but will not be very functional.");
		m_bHaveList = false;
	}
	else
	{
		if (m_fileList.loadPerry(m_strNetworkFilename) && 
			m_Download.Init())
		{
			m_bHaveList = true;
			InitBackgroundData();
			InitImageList();
			
		}
		else
		{
			MessageBox("Could not read list! Program will start but will not be very functional.");
			m_bHaveList = false;
		}
	}

	// Show licence

	if (!AfxGetApp()->GetProfileInt("Settings","ViewLicence",0))
		OnHelpLicence();
	
	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style &= ~(LONG) FWS_ADDTOTITLE;

	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	return TRUE;
}

BOOL CMainFrame::GetText(UINT id, NMHDR *pNMHDR, LRESULT *pResult)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    UINT nID =pNMHDR->idFrom;
    if(nID)
    {
        pTTT->lpszText = MAKEINTRESOURCE(nID);
        pTTT->hinst = AfxGetResourceHandle();
        return(TRUE);
    }
    return(FALSE);
}

void CMainFrame::InitBackgroundData()
{
	ASSERT(m_bHaveList);
	if (!m_pBackgroundData)
		m_pBackgroundData = new CBackgroundData(this, &m_fileList);
}

void CMainFrame::StopBackgroundData()
{
	if (m_pBackgroundData)
		m_pBackgroundData->stopThread();
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnSearchButton() 
{
	m_wndDlgBar.UpdateData(TRUE);
	m_wndOptBar.UpdateData(TRUE);
	
	if (!CheckStatus())
		return;
	
	// Fill info with data
	CSearchInfo info;
	info.m_nSearchType = DAV_SEARCH_STRING;
	info.m_nMaxResultCount = 1000;
	info.m_nMaxFileSize = atoi(m_wndOptBar.m_strMaxFilesize);
	info.m_nMinFileSize = atoi(m_wndOptBar.m_strMinFilesize);
	strcpy((LPSTR)info.m_strQuery, m_wndDlgBar.m_strQuery);

	// Add search to history
	m_History.AddSearch(info);

	StartSearch(info);	
}

void CMainFrame::OnClose() 
{
	StopBackgroundData();
	m_PingThread.StopThread();

	m_Download.Close();

	if (m_bHaveList)
		m_fileList.unLoadPerry();

	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	pDoc->Free();

	CFrameWnd::OnClose();
}

void CMainFrame::OnSearchCommand(WPARAM wParam, LPARAM lParam)
{
	if (wParam == IDU_SEARCH_COMPLETE)
	{
		CDavvesView *pView = (CDavvesView * )GetActiveView();
		CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();

		pView->GetListCtrl().SetItemCountEx(pDoc->m_pSearchResult->GetCount(),LVSICF_NOSCROLL|LVSICF_NOINVALIDATEALL);
		pView->Update();

		if (pDoc->m_SearchInfo.m_nSelectedItem)
			pView->GetListCtrl().EnsureVisible(pDoc->m_SearchInfo.m_nSelectedItem,FALSE);

		CString status;
		if (pDoc->m_pSearchResult->GetTotalCount())
		{
			status.Format("Hits: %d, displaying %d - %d, time: %d ms",
				pDoc->m_pSearchResult->GetTotalCount(),
				pDoc->m_pSearchResult->GetOffset(),
				pDoc->m_pSearchResult->GetCount() + pDoc->m_pSearchResult->GetOffset(),
				pDoc->m_pSearchResult->GetSearchSpan());
		}
		else
		{
			status.Format("Sorry, no hits so I wasted %d ms", pDoc->m_pSearchResult->GetSearchSpan());
			CString strTmp;
			strTmp.Format("No hits found for query '%s'. For search tips, read help!", pDoc->m_pSearchResult->GetSearchQuery());
		}
		m_wndStatusBar.SetWindowText(status);
		
		m_bNext = m_History.CanMoveForward();
		m_bPrev = m_History.CanMoveBack();	
		m_bDown = (pDoc->m_pSearchResult->GetSize() + pDoc->m_pSearchResult->GetOffset()) < pDoc->m_pSearchResult->GetTotalCount();
		m_bUp = pDoc->m_pSearchResult->GetOffset() != 0;

		m_wndDlgBar.m_ctrlSearchButton.SetWindowText("Go");
	}
	else if (wParam == IDU_SEARCH_ABORT)
	{
		CDavvesView *pView = (CDavvesView * )GetActiveView();
		CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();

		pView->GetListCtrl().SetItemCount(0);
		pView->Update();

		CString status;
		status.Format("Search aborted!");
		m_wndStatusBar.SetWindowText(status);

		m_bNext = m_History.CanMoveForward();
		m_bPrev = m_History.CanMoveBack();	
		m_bDown = (pDoc->m_pSearchResult->GetSize() + pDoc->m_pSearchResult->GetOffset()) < pDoc->m_pSearchResult->GetTotalCount();
		m_bUp = pDoc->m_pSearchResult->GetOffset() != 0;

		m_wndDlgBar.m_ctrlSearchButton.SetWindowText("Go");		
	}
}

void CMainFrame::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& list = pView->GetListCtrl();
	
	CRect rect;
	list.GetWindowRect(rect);

	if (rect.PtInRect(point)) // it's for the list ctrl
	{
		CMenu popupMenus;
		if (popupMenus.LoadMenu(IDR_POPUP_MENU))
		{
			POSITION pos = list.GetFirstSelectedItemPosition();
			CMenu* pPopup;
			if (pos == NULL)
			{
				pPopup = popupMenus.GetSubMenu(1);
			}
			else
			{
				UINT nSelectCount = list.GetSelectedCount();
				if (nSelectCount == 1)
					pPopup = popupMenus.GetSubMenu(0);
				else
					pPopup = popupMenus.GetSubMenu(3);
			}
			pPopup->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON,point.x,point.y,this,NULL);	
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Item commands
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::OnItemAddtoqueue() 
{
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CListCtrl& list = pView->GetListCtrl();
	
	POSITION pos = list.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		TRACE0("No items were selected!\n");
	}
	else
	{
		CDownloadGroup group;
		CDownloadFile* file;
		while (pos)
		{
			int nItem = list.GetNextSelectedItem(pos);
			file = pDoc->m_pSearchResult->GetAt(nItem);
			group.AddFile(file);
		}			
		m_Download.AddGroup(&group);
		m_Download.Download();
	}
}

void CMainFrame::OnItemOpendirectory() 
{
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& list = pView->GetListCtrl();

	POSITION pos = list.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		TRACE0("No items were selected!\n");
	}
	else
	{
		int nItem = list.GetNextSelectedItem(pos);
		CDownloadFile* file = pDoc->m_pSearchResult->GetAt(nItem);
		CString strPath;
		m_fileList.getDirectoryName(strPath,file->m_nFileId);
		int nReturn = (int)ShellExecute(m_hWnd, "open", strPath, NULL, NULL, SW_SHOWNORMAL);
		switch (nReturn)
		{
		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			MessageBox("The path/file do not exist. Update your list often to minimize this event.","Davves");
			break;
		case SE_ERR_ACCESSDENIED:
			MessageBox("Access denied. Update your list often to minimize this event.","Davves");
			break;
		}
	}	
}

void CMainFrame::OnItemExecute() 
{
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& list = pView->GetListCtrl();

	POSITION pos = list.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		TRACE0("No items were selected!\n");
	}
	else
	{
		int nItem = list.GetNextSelectedItem(pos);
		CDownloadFile* file = pDoc->m_pSearchResult->GetAt(nItem);
		CString strFilename, strPath;
		m_fileList.getDirectoryName(strPath,file->m_nFileId);
		m_fileList.getFilename(strFilename,file->m_nFileId);
		CString strFullPath;
		strFullPath.Format("%s%s",strPath,strFilename);
		int nReturn = (int)ShellExecute(m_hWnd, "open", strFullPath, NULL, NULL, SW_SHOWNORMAL);
		switch (nReturn)
		{
		case ERROR_FILE_NOT_FOUND:
		case ERROR_PATH_NOT_FOUND:
			MessageBox("The path/file do not exist. Update your list often to minimize this event.","Davves");
			break;
		case SE_ERR_ACCESSDENIED:
			MessageBox("Access denied. Update your list often to minimize this event.","Davves");
			break;
				
		}
	}	
}


void CMainFrame::OnItemListdirectory() 
{
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& list = pView->GetListCtrl();

	POSITION pos = list.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		TRACE0("No items were selected!\n");
	}
	else
	{
		m_wndDlgBar.UpdateData(TRUE);

		int nItem = list.GetNextSelectedItem(pos);
		CDownloadFile* file = pDoc->m_pSearchResult->GetAt(nItem);
	
		// Save pos
		m_History.SavePosition(nItem);

		CSearchInfo info;
		info.m_nSearchType = DAV_SEARCH_DIR;
		info.m_nDirId = m_fileList.getDirectoryId(file->m_nFileId);
		info.m_nMaxResultCount = 1000;
		m_History.AddSearch(info);
		StartSearch(info);
	}	
}

void CMainFrame::OnItemPing() 
{
	if (m_PingThread.IsRunning())
		return;

	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& listCtrl = pView->GetListCtrl();

	CIntArray* arPing = new CIntArray();
	arPing->SetSize(listCtrl.GetSelectedCount());

	POSITION pos = listCtrl.GetFirstSelectedItemPosition();
	for (UINT i=0;i<listCtrl.GetSelectedCount();i++)
	{
		arPing->SetAt(i,listCtrl.GetNextSelectedItem(pos));
	}
	
	m_PingThread.SetDocument(pDoc);
	m_PingThread.SetView(pView);
	m_PingThread.SetPingArray(arPing);
	m_PingThread.m_pFileList = &m_fileList;
	m_PingThread.StartThread();
}

void CMainFrame::OnItemCopypath() 
{
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CListCtrl& listCtrl = pView->GetListCtrl();

	POSITION pos = listCtrl.GetFirstSelectedItemPosition();
	for (UINT i=0;i<listCtrl.GetSelectedCount();i++)
	{
		int nItem = listCtrl.GetNextSelectedItem(pos);
		CDownloadFile* file = pDoc->m_pSearchResult->GetAt(nItem);
		//put your text in source
		if(OpenClipboard())
		{
			HGLOBAL clipbuffer;
			char * buffer;
			EmptyClipboard();
			clipbuffer = GlobalAlloc(GMEM_DDESHARE, file->m_nFilenameLength+file->m_nSharenameLength+1);
			buffer = (char*)GlobalLock(clipbuffer);

			memcpy(buffer, file->m_strSharename, file->m_nSharenameLength);
			buffer += file->m_nSharenameLength;
			memcpy(buffer, file->m_strFilename, file->m_nFilenameLength);
			buffer[file->m_nFilenameLength] = '\0';			

			GlobalUnlock(clipbuffer);
			SetClipboardData(CF_TEXT,clipbuffer);
			CloseClipboard();
		}
	}
}

void CMainFrame::OnListUpdate() 
{
/*	CDavvesView* pView = (CDavvesView *)GetActiveView();

	pView->ResetContent();
	pView->Invalidate(FALSE);
	pView->LockWindowUpdate();

	StopBackgroundData();

	if (GetNewList())
		m_bHaveList = m_fileList.loadPerry(m_strNetworkFilename);
	else
		MessageBox("Load failed!\nTry fetch list again!");

	InitImageList();
	pView->UpdateImageList();

	if (m_bHaveList)
		InitBackgroundData();

//	pView->SetNoItemsText("New list loaded\n\nThere are no items to show in this view");
	pView->UnlockWindowUpdate();
	*/
}

void CMainFrame::OnItemAddasseparategroups() 
{
/*	CDavvesView* pView = (CDavvesView *)GetActiveView();
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	CListCtrl& list = pView->GetListCtrl();
	CDownloadDlg* pDlg = (CDownloadDlg *)m_pDownloadThread->m_pMainWnd;
	
	POSITION pos = list.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		TRACE0("No items were selected!\n");
	}
	else
	{
		/// DirDialog
		CDirDialog dirDlg;
		if (dirDlg.DoModal() == IDOK)
		{
			CString strDir = dirDlg.GetPath();
			while (pos)
			{
	
				int nItem = list.GetNextSelectedItem(pos);
				CDownloadFile* file = pDoc->m_pSearchResult->GetAt(nItem);
				CString filename;
				m_fileList.getFilename(filename,file->m_nFileId);
				
				CString pathname;
				pathname.Format("%s%s",strDir,filename);
				
				
				CDownloadQueuedFile* qFile = new CDownloadQueuedFile(file);  // deleted by deconstructor in CDownloadGroup
				CDownloadGroup* group = new CDownloadGroup;	// deleted by deconstructor in CDownloadGroups
				group->AddFile(qFile);

				group->SetPathname(pathname);
				
				// Enqueue
				// group->SetPostDownloadVerb("enqueue");

				file = pDoc->m_pSearchResult->GetAt(nItem);
				qFile = new CDownloadQueuedFile(file);
				group->AddFile(qFile);

				pDlg->AddGroup(group);
			}
		}
	}	
	*/
}

//////////////////////////////////////////////////////////////////////////////
// Toolbar buttons
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::OnUp() 
{
	if (!CheckStatus())
		return;

	int nOldMax = 1000;
	m_wndDlgBar.UpdateData(TRUE);
	UINT nMaxHits = 1000;

	CSearchInfo info;
	info.m_nSearchType = DAV_SEARCH_BROWSE;
	info.m_nMaxResultCount = nMaxHits;
	info.m_nOffsetMove = -1* nOldMax;
	StartSearch(info);
}



void CMainFrame::OnDown() 
{
	if (!CheckStatus())
		return;

	int nOldMax = 1000;
	m_wndDlgBar.UpdateData(TRUE);
	UINT nMaxHits = 1000;

	CSearchInfo info;
	info.m_nSearchType = DAV_SEARCH_BROWSE;
	info.m_nMaxResultCount = nMaxHits;
	info.m_nOffsetMove = nOldMax;
	StartSearch(info);	
}

void CMainFrame::OnNextButton()
{
	if (!m_History.CanMoveForward())
		return;

	StartSearch(m_History.MoveForward());
}

void CMainFrame::OnBackButton()
{
	if (!m_History.CanMoveBack())
		return;

	StartSearch(m_History.MoveBack());
}

void CMainFrame::InitImageList()
{
	m_ImageList.DeleteImageList();
	m_ImageList.Create(16,16,ILC_COLOR32|ILC_MASK,m_fileList.GetFileTypeCount(),1);

	for (UINT i=0;i<m_fileList.GetFileTypeCount();i++)
	{
		SHFILEINFO sfi;
		SHGetFileInfo((LPSTR)m_fileList.GetFileType(i),
              FILE_ATTRIBUTE_NORMAL,
              &sfi,
              sizeof(SHFILEINFO), 
              SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | SHGFI_TYPENAME);
		m_ImageList.Add(sfi.hIcon);
	}
}

CImageList* CMainFrame::GetImageList()
{
	return &m_ImageList;
}

void CMainFrame::OnSearchQueryButton()
{
	BYTE hi = -1;
	BYTE lo = 0;

	WORD sel = hi << 8;
	sel += lo;
	m_wndDlgBar.m_ctrlQueryEdit.SetFocus();
	m_wndDlgBar.m_ctrlQueryEdit.SetSel(sel,FALSE);
}

////////////////////////////////////////////////////////////////////////
//	Statusbar Indicators
////////////////////////////////////////////////////////////////////////

void CMainFrame::OnUpdateListCreatedIndicator(CCmdUI *pCmdUI)
{
	if (m_bHaveList)
	{
		char strDate[17];
		m_fileList.getCreated(strDate);
		pCmdUI->SetText(strDate);
	}
	else
	{
		pCmdUI->SetText("List not loaded");
	}
}

void CMainFrame::OnUpdateListDataIndicator(CCmdUI *pCmdUI)
{
	if (m_bHaveList)
	{
		CListInfo info;
		m_fileList.GetInfo(info);
		CString strListData;
		
		strListData.Format(IDS_LIST_DATA, info.m_nTotalFileCount, info.m_nTotalFilesize, info.m_nTotalUniqueFiles, info.m_nTotalUniqueSize);
			
		pCmdUI->SetText(strListData);
	}
	else
	{
		pCmdUI->SetText("");
	}
}

////////////////////////////////////////////////////////////////////////
//	Help/Licence stuff
////////////////////////////////////////////////////////////////////////

void CMainFrame::OnWebHelp()
{
	CString strPath("http://p13.ryd.student.liu.se/davves/help.php");
	int nReturn = (int)ShellExecute(m_hWnd, "open", strPath, NULL, NULL, SW_SHOWNORMAL);
}


void CMainFrame::OnHelpCheckversion() 
{
	CString strPath;
	strPath.Format("http://p13.ryd.student.liu.se/davves/version.php?major=%d&minor=%d&branch=%d", 
		DAV_PROGRAM_VERSION_MAJOR,
		DAV_PROGRAM_VERSION_MINOR,
		DAV_PROGRAM_VERSION_BRANCH);
	int nReturn = (int)ShellExecute(m_hWnd, "open", strPath, NULL, NULL, SW_SHOWNORMAL);
}

void CMainFrame::OnHelpLicence() 
{
	CFile licenceFile;
	CFileStatus licenceStatus;
	CString strLicence;

	if (licenceFile.Open("COPYING.txt",CFile::modeRead,NULL))
	{
		char buf[DAV_HELP_BUFFER_SIZE+1];

		int nRead;
		do
		{
			nRead = licenceFile.Read(buf,DAV_HELP_BUFFER_SIZE);
			buf[nRead] = '\0';
			strLicence += (LPSTR)buf;
		} while (nRead);
		licenceFile.Close();
	}
	else
	{
		strLicence = "Couldn't open licence file!\n";
	}

	CTextDlg dlg;
	dlg.SetText(strLicence);
	dlg.m_bMode = false;
	dlg.m_bDisable = AfxGetApp()->GetProfileInt("Settings","ViewLicence",0);
	dlg.DoModal();
	AfxGetApp()->WriteProfileInt("Settings","ViewLicence",dlg.m_bDisable);
}

////////////////////////////////////////////////////////////////////////
//	Syscommand
////////////////////////////////////////////////////////////////////////

void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam) 
{
/*	CDownloadDlg* pDlg = (CDownloadDlg *)m_pDownloadThread->m_pMainWnd;

	switch (nID)
	{
	case SC_CLOSE:
		if (pDlg->GetDownloadCount())
		{
			int nRet = MessageBox("You have unfinished downloads!\nDo you want to abort those?","Warning!",MB_OKCANCEL);
			if (nRet == IDOK)
				CFrameWnd::OnSysCommand(nID, lParam);
		}
		else
			CFrameWnd::OnSysCommand(nID, lParam);

		break;
	default:
		CFrameWnd::OnSysCommand(nID, lParam);
		break;
	}
	*/
	CFrameWnd::OnSysCommand(nID, lParam);
}

//////////////////////////////////////////////////////////////////////////////
// Menu commands
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::OnViewDownload() 
{
/*	CDownloadDlg* pDlg = (CDownloadDlg *)m_pDownloadThread->m_pMainWnd;
	pDlg->ShowWindow(TRUE);*/
}

void CMainFrame::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnItemAddtoqueue();		
	CFrameWnd::OnLButtonDblClk(nFlags, point);
}

//////////////////////////////////////////////////////////////////////////////
// Options / Tools
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::OnFileOptions() 
{
	CSAPrefsDialog dlg;

	// the "pages" (all derived from CSAPrefsSubDlg)
	COptionsPaths pagePaths;
	COptionsUI pageUI;
	COptionsDownload pageDownload;

	// add 3 pages
	dlg.AddPage(pageDownload,"Download Settings");
	dlg.AddPage(pagePaths, "Paths",&pageDownload);
	dlg.AddPage(pageUI,"User interface",&pageDownload);

	// the prefs dialog title
	dlg.SetTitle("Download options");
	
	// text drawn on the right side of the shaded
	// page label. this does not change when the
	// pages change, hence "constant".
	dlg.SetConstantText("Davves");
	
	// Setup paths
	CWinApp* app = AfxGetApp();
	pageDownload.m_strMaxDownloadCount.Format("%d",m_Settings.m_nMaxDownloadCount);
	pagePaths.m_strDefaultPath = m_Settings.m_strDefaultDownloadPath;	
	pageUI.m_bAskBeforeDelete = m_Settings.m_bAskBeforeDelete;
	pageUI.m_bAskForPath = m_Settings.m_bAskForFilename;
	if (dlg.DoModal()==IDOK)
	{
		m_Settings.m_strDefaultDownloadPath = pagePaths.m_strDefaultPath;
		
		m_Settings.m_bAskBeforeDelete = pageUI.m_bAskBeforeDelete;
		m_Settings.m_bAskForFilename = pageUI.m_bAskForPath;
		m_Settings.m_nMaxDownloadCount = atoi(pageDownload.m_strMaxDownloadCount);

		m_Settings.WriteToReg();
	}

}


void CMainFrame::OnToolsCheckupdates() 
{
/*	CDownloadDlg* pDlg = (CDownloadDlg *)m_pDownloadThread->m_pMainWnd;
	if (pDlg->GetDownloadCount())
	{
		MessageBox("You must finish downloads before updating program","Davves");
		return;
	}

	CUpdate update;
	int nRet = update.DoModal();	
	if (nRet == IDOK)
	{
		PostMessage(WM_CLOSE,0,0);
	}*/
}


//////////////////////////////////////////////////////////////////////////////
// Internal/private helpers
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::StartSearch(CSearchInfo info)
{
	m_bNext = false;
	m_bPrev = false;
	m_bUp = false;
	m_bDown = false;
	
	// Make sure that we're not searching any more
	m_pBackgroundData->stopThread();
	m_PingThread.StopThread();

	// Show that we're searching
	m_wndDlgBar.m_ctrlSearchButton.SetWindowText("Stop");
	m_wndStatusBar.SetWindowText("Searching...");

	// Reset and prepare document
	CDavvesDoc* pDoc = (CDavvesDoc *)GetActiveDocument();
	pDoc->Free();
	pDoc->Allocate(info.m_nMaxResultCount);

	// Reset view
	((CDavvesView *)GetActiveView())->GetListCtrl().SetItemCount(0);
	if (info.m_nSearchType == DAV_SEARCH_STRING)
		m_wndDlgBar.m_strQuery.Format("%s",info.m_strQuery);

	// Update bars
	m_wndOptBar.m_strMaxFilesize.Format("%d",info.m_nMaxFileSize);
	m_wndOptBar.m_strMinFilesize.Format("%d",info.m_nMinFileSize);

	m_wndDlgBar.UpdateData(FALSE);
	m_wndOptBar.UpdateData(FALSE);
	
	// Update/Prepare thread with new environment
	m_pBackgroundData->SetSearchInfo(info);
	m_pBackgroundData->setListView((CDavvesView *)GetActiveView());
	m_pBackgroundData->setListDoc(pDoc);

	// Start search!
	m_pBackgroundData->startThread();
}

bool CMainFrame::CheckStatus()
{
	if (!m_bHaveList)
	{
		m_wndStatusBar.SetWindowText("List not loaded.");
		return false;
	}
	else if (m_wndDlgBar.m_strQuery.IsEmpty())
	{
		m_wndStatusBar.SetWindowText("Search query too short!");
		return false;
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////////
// UI Updaters
//////////////////////////////////////////////////////////////////////////////

void CMainFrame::OnUpdateDown(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bDown);	
}

void CMainFrame::OnUpdateUp(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bUp);
}

void CMainFrame::OnUpdateNextButton(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bNext);	
}

void CMainFrame::OnUpdateBackButton(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bPrev);
}

LRESULT CMainFrame::OnCompletion(WPARAM wParam, LPARAM lParam)
{
	m_Download.m_bDLProcRunning = false;
	::CloseHandle((HANDLE)lParam);
	return 0;
}

