 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// PingThread.cpp: implementation of the CPingThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "davves.h"
#include "FileList.h"
#include "DavvesDoc.h"
#include "DavvesView.h"
#include "PingThread.h"
#include "ping.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPingThread::CPingThread()
{
	m_bContinue = TRUE;
	m_bIsRunning = FALSE;
	m_pArPing = NULL;
}

CPingThread::~CPingThread()
{

}

//////////////////////////////////////////////////////////////////////
// Setters
//////////////////////////////////////////////////////////////////////

void CPingThread::SetDocument(CDavvesDoc *pDoc)
{
	m_pDoc = pDoc;
}

void CPingThread::SetView(CDavvesView* pView)
{
	m_pView = pView;
}

void CPingThread::SetPingArray(CIntArray* pArPing)
{
	m_pArPing = pArPing;
}

//////////////////////////////////////////////////////////////////////
// Thread proc
//////////////////////////////////////////////////////////////////////

UINT CPingThread::PingThreadProc(LPVOID pParam)
{
	CPingThread* pData = (CPingThread *)pParam;		
	pData->m_bContinue = TRUE;
	pData->m_bIsRunning = TRUE;

	if (!pData->DoPinging())
		return 1;

	delete pData->m_pArPing;
	
	pData->m_bIsRunning = FALSE;
	return 0;	
}

UINT CPingThread::PingHost(LPVOID pParam)
{
	CPing ping;
	CPingReply reply;
	CDownloadShare* pShare = (CDownloadShare* )pParam;

	CString ip;
	ip.Format("%d.%d.%d.%d",DG_IP_A,DG_IP_B,pShare->m_nIP1,pShare->m_nIP2);

	if (ping.Ping1(ip,reply,20,600))
	{
		pShare->m_nPing = reply.RTT;
		TRACE("Ping reply from %s: %d\n",ip,reply.RTT);
	}
	else
	{
		pShare->m_nPing = -2;
		TRACE("No ping reply from %s\n",ip);
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Status check
//////////////////////////////////////////////////////////////////////

bool CPingThread::IsRunning()
{
	return m_bIsRunning;
}

//////////////////////////////////////////////////////////////////////
// Control procs
//////////////////////////////////////////////////////////////////////

void CPingThread::Reset()
{
	StopThread();
	StartThread();
}

bool CPingThread::StartThread()
{
	ASSERT(m_pArPing);
	if (StopThread())
	{
		m_thread = AfxBeginThread(PingThreadProc,this,THREAD_PRIORITY_LOWEST);
		return true;
	}
	else
	{
		return false;
	}
}

bool CPingThread::StopThread()
{
	if (IsRunning())
	{
		m_bContinue = false;
		m_thread->m_bAutoDelete = FALSE;
		WaitForSingleObject(m_thread->m_hThread,INFINITE);
		delete m_thread;
	}
	return true;
}

//////////////////////////////////////////////////////////////////////
// Main proc
//////////////////////////////////////////////////////////////////////

bool CPingThread::DoPinging()
{
	CSearchResult* pList = m_pDoc->m_pSearchResult;
	CListCtrl& pListCtrl = m_pView->GetListCtrl();

	int nIndex = 0;
	for (int i=0;i<m_pArPing->GetSize();i++)
	{
		nIndex = m_pArPing->GetAt(i);
		CDownloadFile* pFile = pList->GetAt(nIndex);

		AfxBeginThread(PingHost,(LPVOID)pFile->m_pShare);
	} 
	
	pListCtrl.Invalidate();
	return true;
}


