 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// SearchResult.cpp: implementation of the CSearchResult class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "davves.h"
#include "SearchResult.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

extern unsigned char UPPERCHAR[];


CSearchResult::CSearchResult(int nFiles)
{
	m_arFiles = new CDownloadFile*[nFiles];
	memset(m_arFiles,0,nFiles*sizeof(CDownloadFile*));
	m_nFiles = nFiles;
	m_nSearchSpan = 0;
	m_nOffset = 0;
	m_nMaxId = -1;
	m_nTotalCount = 0;
	strcpy((char *)m_strSearchQuery,"");
}

CSearchResult::~CSearchResult()
{
	Free();
}

void CSearchResult::Free()
{
	if (m_arFiles)
	{
		for (int i=0;i<m_nFiles;i++)
			delete m_arFiles[i];

		delete m_arFiles;	
		m_arFiles = NULL;
	}
}


void CSearchResult::SetSearchQuery(const unsigned char str[])
{
	ASSERT(strlen((char *)str)<DAV_MAX_SEARCH_QUERY);
	strcpy((char *)m_strSearchQuery,(char *) str);
}

//////////////////////////////////////////////////////////////////////
// setters
//////////////////////////////////////////////////////////////////////

CString CSearchResult::GetSearchQuery()
{
	return m_strSearchQuery;
}

bool CSearchResult::SetAt(int nId, CDownloadFile* file)
{
	ASSERT(nId<m_nFiles);
	m_nMaxId = __max(m_nMaxId,nId);
	m_arFiles[nId] = file;
	return true;
}

void CSearchResult::SetSearchSpan(UINT ms)
{
	m_nSearchSpan = ms;
}

void CSearchResult::SetOffset(int nOffset)
{
	m_nOffset = nOffset;
}

void CSearchResult::SetTotalCount(int nTotalCount)
{
	m_nTotalCount = nTotalCount;
}

//////////////////////////////////////////////////////////////////////
// getters
//////////////////////////////////////////////////////////////////////

CDownloadFile* CSearchResult::GetAt(int nId)
{
	ASSERT(m_arFiles[nId]);
	return m_arFiles[nId];
}

int CSearchResult::GetTotalCount()
{	
	return m_nTotalCount;
}

int CSearchResult::GetCount()
{	
	return m_nMaxId + 1;
}

int CSearchResult::GetOffset()
{
	return m_nOffset;
}

int CSearchResult::GetUpperBound()
{
	return m_nMaxId;
}

int CSearchResult::GetSearchSpan()
{
	return m_nSearchSpan;
}

int CSearchResult::GetSize()
{	
	return m_nFiles;
}

//////////////////////////////////////////////////////////////////////
// sorting
//////////////////////////////////////////////////////////////////////

void CSearchResult::Sort(UINT nCol, UINT nOrder)
{
	if (!m_nTotalCount || !m_nMaxId)
		return;

	switch (nCol)
	{
	case DAV_FILE_FILENAME:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpFilenamesAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpFilenamesDesc);
		else
			QSort(0,m_nMaxId,CmpFilenamesOrig);
		break;
	case DAV_FILE_FILESIZE:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpFilesizeAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpFilesizeDesc);
		break;
	case DAV_FILE_SHARENAME:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpSharenamesAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpSharenamesDesc);
		break;
	case DAV_FILE_PING:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpPingAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpPingDesc);
		break;
	case DAV_FILE_DISTANCE:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpDistanceAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpDistanceDesc);
		break;
	case DAV_FILE_ACCESS:
		if (nOrder == DAV_SORT_ASCE)
			QSort(0,m_nMaxId,CmpAccessAsce);
		else if (nOrder == DAV_SORT_DESC)
			QSort(0,m_nMaxId,CmpAccessDesc);
		break;
	}
}

void CSearchResult::QSort(int l, int r, CFT cmp)
{
	int i, j;
	CDownloadFile* v;
	
	Sort3(l,(l+r)/2,r, cmp);
	if ((r-l)>2)
	{
		Swap(l+1,(l+r)/2);
		i = l+1;
		j = r;
		v = m_arFiles[l+1];  // use as pivot

		while (i<j)
		{
			i++;
			while (cmp(m_arFiles[i],v) < 0)
			{
				i++;
			}
			j--;
			while (cmp(m_arFiles[j],v) > 0)
			{
				j--;
			}
			Swap(i,j);
		}
		Swap(i,j);
		Swap(j,l+1);

		QSort(l,j-1,cmp);
		QSort(j+1,r,cmp);
	}
}

void CSearchResult::Sort3(const int x, const int y, const int z, CFT cmp)
{
	ASSERT(x<=m_nMaxId && y<=m_nMaxId && z<=m_nMaxId);

	if (cmp(m_arFiles[x],m_arFiles[z]) > 0)
		Swap(x,z);

	if (cmp(m_arFiles[x],m_arFiles[y]) > 0)
		Swap(x,y);

	if (cmp(m_arFiles[y],m_arFiles[z]) > 0)
		Swap(y,z);
}

int CSearchResult::CmpFilenamesAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return _strnicmp((LPSTR)file1->m_strFilename,(LPSTR)file2->m_strFilename,__min(file1->m_nFilenameLength,file2->m_nFilenameLength));
}

int CSearchResult::CmpFilenamesDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return (-1 * _strnicmp((LPSTR)file1->m_strFilename,(LPSTR)file2->m_strFilename,__min(file1->m_nFilenameLength,file2->m_nFilenameLength)));
}

int CSearchResult::CmpFilenamesOrig(const CDownloadFile *file1, const CDownloadFile *file2)
{
	if (file1->m_nFileId>file2->m_nFileId)
		return 1;
	else if (file1->m_nFileId<file2->m_nFileId)
		return -1;
	else
		return 0;
}

int CSearchResult::CmpFilesizeAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file1->m_nFilesize - file2->m_nFilesize;
}

int CSearchResult::CmpFilesizeDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file2->m_nFilesize - file1->m_nFilesize;
}

int CSearchResult::CmpDistanceAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file1->m_pShare->m_nDistance - file2->m_pShare->m_nDistance;
}

int CSearchResult::CmpDistanceDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file2->m_pShare->m_nDistance - file1->m_pShare->m_nDistance;
}

int CSearchResult::CmpAccessAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file1->m_pShare->m_nAccessHistory - file2->m_pShare->m_nAccessHistory;
}

int CSearchResult::CmpAccessDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return file2->m_pShare->m_nAccessHistory - file1->m_pShare->m_nAccessHistory;
}

int CSearchResult::CmpSharenamesAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return _strnicmp((LPSTR)file1->m_strSharename,(LPSTR)file2->m_strSharename,__min(file1->m_nSharenameLength,file2->m_nSharenameLength));
}

int CSearchResult::CmpSharenamesDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	return (-1 * _strnicmp((LPSTR)file1->m_strSharename,(LPSTR)file2->m_strSharename,__min(file1->m_nSharenameLength,file2->m_nSharenameLength)));
}

int CSearchResult::CmpPingAsce(const CDownloadFile *file1, const CDownloadFile *file2)
{
	int nPing1 = file1->m_pShare->m_nPing;
	int nPing2 = file2->m_pShare->m_nPing;

	if (nPing1>=0 && nPing2>=0)
		return nPing2 - nPing1;
	else if (nPing1 >= 0)
		return 1;
	else if (nPing2 >= 0)
		return -1;
	else if (nPing1 == -1 && nPing2 == -2)
		return 1;
	else if (nPing2 == -1 && nPing1 == -2)
		return -1;
	else
		return 0;
}

int CSearchResult::CmpPingDesc(const CDownloadFile *file1, const CDownloadFile *file2)
{
	int nPing1 = file1->m_pShare->m_nPing;
	int nPing2 = file2->m_pShare->m_nPing;

	if (nPing1>=0 && nPing2>=0)
		return nPing1 - nPing2;
	else if (nPing1 >= 0)
		return -1;
	else if (nPing2 >= 0)
		return 1;
	else if (nPing1 == -1 && nPing2 == -2)
		return -1;
	else if (nPing2 == -1 && nPing1 == -2)
		return 1;
	else
		return 0;
}

void CSearchResult::Swap(int idx1, int idx2)
{
	ASSERT(idx1<=m_nMaxId && idx2<=m_nMaxId);
	CDownloadFile* tmp;
	tmp = m_arFiles[idx1];
	m_arFiles[idx1] = m_arFiles[idx2];
	m_arFiles[idx2] = tmp;
}

int CSearchResult::GetItemOfStr(LPTSTR str,int nStart)
{
	int nIndex = 0;
	
	for (int i=0;i<m_nMaxId;i++)
	{
		nIndex = (i+nStart)%m_nMaxId;
		if (strnicmp(str,(char *)m_arFiles[nIndex]->m_strFilename,strlen(str)) == 0)
			return nIndex;
	}

	return -1;
}
