// SearchResult.h: interface for the CSearchResult class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SEARCHRESULT_H__BE1526E6_0245_4C43_8A67_FB9720FFF2D9__INCLUDED_)
#define AFX_SEARCHRESULT_H__BE1526E6_0245_4C43_8A67_FB9720FFF2D9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "afxtempl.h"
#include "FileList.h"

 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/


// Search defines
#define DAV_SORT_ASCE 0
#define DAV_SORT_DESC 1
#define DAV_SORT_ORIG 2
#define DAV_SORT_ORDERS 3

typedef int (__cdecl *CFT)(const CDownloadFile*, const CDownloadFile*);
typedef void (__cdecl *S3)(CDownloadFile* ar, const int x, const int y, const int z);

class CSearchResult
{
private:
	unsigned char m_strSearchQuery[DAV_MAX_SEARCH_QUERY];
	static int CmpSharenamesAsce(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpSharenamesDesc(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpFilenamesAsce(const CDownloadFile* file1, const CDownloadFile* file2);
	static int CmpFilenamesDesc(const CDownloadFile* file1, const CDownloadFile* file2);
	static int CmpFilenamesOrig(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpFilesizeAsce(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpFilesizeDesc(const CDownloadFile *file1, const CDownloadFile *file2);	
	static int CmpDistanceAsce(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpDistanceDesc(const CDownloadFile *file1, const CDownloadFile *file2);	
	static int CmpPingDesc(const CDownloadFile *file1, const CDownloadFile *file2);	
	static int CmpPingAsce(const CDownloadFile *file1, const CDownloadFile *file2);	
	static int CmpAccessDesc(const CDownloadFile *file1, const CDownloadFile *file2);
	static int CmpAccessAsce(const CDownloadFile *file1, const CDownloadFile *file2);
	
	
	void Sort3(const int x, const int y, const int z, CFT cmp);
	void QSort(int l, int r, CFT cmp);
	volatile int m_nMaxId;
	CDownloadFile** m_arFiles;
	int m_nFiles;
	UINT m_nSearchSpan;
	int m_nOffset;
	int m_nTotalCount;

	// privates for sorting
	void Swap(int idx1, int idx2);

public:
	int GetItemOfStr(LPTSTR str,int nStart);
	void SetTotalCount(int nTotalCount);
	void SetOffset(int nOffset);
	// Construction
	CSearchResult(int nFiles);
	~CSearchResult();
	void Free();

	// Modify
	bool Add(CDownloadFile* file);
	bool SetAt(int nId, CDownloadFile* file);
	void Sort(UINT nCol, UINT nOrder);
	void SetSearchSpan(UINT ms);
	void SetSearchQuery(const unsigned char str[]);

	// Read
	CString GetSearchQuery();
	int GetTotalCount();
	int GetCount();
	int GetOffset();
	int GetSize();
	CDownloadFile* GetAt(int nId);	
	int GetUpperBound();
	int GetSearchSpan(); // Returns search time in milli seconds
};

#endif // !defined(AFX_SEARCHRESULT_H__BE1526E6_0245_4C43_8A67_FB9720FFF2D9__INCLUDED_)
