// ControlFile.cpp: implementation of the CControlFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "download.h"
#include "ControlFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const LPCTSTR strFileOnDisk= ".\\download.dat";
const DWORD dwIPCFileSize = 64 * 1024;
const LPCTSTR strDiskFileName = _T("45EE00B1-AE22-47e8-89B5-CD91699E8711");
const LPCTSTR strIPCMapName = _T("BF74C014-49B7-4564-A554-B83C1B2E136B");
const LPCTSTR sReadWriteLockName = _T("1D57E964-A516-48ba-9AB3-D01DB4D50EA4");
const DWORD dwMaxWait = 200;

CControlFile::CControlFile()
{
	m_bLoaded = false;

	m_pViewOfIPC = NULL;
	m_pLock = NULL;
}

CControlFile::~CControlFile()
{
	if (m_bLoaded)
		Unload();
}

void CControlFile::AddGroup(CDownloadGroup* group)
{
	ASSERT(m_bLoaded);
	m_lsGroups.AddTail(group);
	Rewind();
}

bool CControlFile::Load()
{
	CFile fileDisk;
	CFileStatus stat;
	HANDLE hDiskMap;
	BYTE* pView;

	if (fileDisk.Open(strFileOnDisk,CFile::modeRead|CFile::shareExclusive,NULL))
	{	
		fileDisk.GetStatus(stat);

		hDiskMap = CreateFileMapping((void *)fileDisk.m_hFile,NULL,PAGE_READONLY,0,stat.m_size,strDiskFileName);
		if (hDiskMap != NULL) 
		{ 
			pView = (BYTE*)MapViewOfFile(hDiskMap, FILE_MAP_READ, 0, 0, 0);
			if (pView)
			{
				LoadBlocks(pView);	
				UnmapViewOfFile(pView);	
			}
			CloseHandle(hDiskMap);
		}
		fileDisk.Close();
	}
	else
		TRACE("Error opening file. Skipping load!");
	
	// close mapping
	
	

	// Done with loading from disk... Now set up IPC

	// create the Single Writer/Multiple Reader Guard
    m_pLock = new CIPCReadWriteLock(
        sReadWriteLockName,
        dwMaxWait);
    
	// Create file mapping which can contain dwMemoryFileSize characters
    m_hIPCMap = CreateFileMapping(
        INVALID_HANDLE_VALUE,           // system paging file
        NULL,                           // security attributes
        PAGE_READWRITE,                 // protection
        0,                              // high-order DWORD of size
        dwIPCFileSize*sizeof(TCHAR), // low-order DWORD of size
        strIPCMapName);					// name

    DWORD dwError = GetLastError();     // if ERROR_ALREADY_EXISTS 
            // this instance is not first (other instance created file mapping)

    if (!m_hIPCMap)
    {
		Unload();
        return false;
    }
    else
    {
        m_pViewOfIPC = (BYTE *)MapViewOfFile(
            m_hIPCMap,             // handle to file-mapping object
            FILE_MAP_ALL_ACCESS,        // desired access
            0,
            0,
            0);                         // map all file

        if (!m_pViewOfIPC)
        {
            return false;
        }
	}

    if ( dwError == ERROR_ALREADY_EXISTS )
    {
	}

	m_bLoaded = true;

	return true;
}


BOOL CControlFile::Save()
{
	CFile fileDisk;
	HANDLE hDiskMap;
	BYTE* pView;
	
	if (!fileDisk.Open(strFileOnDisk,CFile::modeReadWrite|CFile::shareExclusive|CFile::modeCreate,NULL))
		return FALSE;

	int nSize = PrepareData();

	hDiskMap = CreateFileMapping((void *)fileDisk.m_hFile,NULL,PAGE_READWRITE,0,nSize,strDiskFileName);
	if (hDiskMap == NULL || GetLastError() == ERROR_ALREADY_EXISTS) 
	{ 
		TRACE("Fet Error");
	}

	pView = (BYTE *)MapViewOfFile(hDiskMap, FILE_MAP_WRITE, 0, 0, 0); 
	if (!pView)
	{
		TRACE("Fet Error");
	}

	WriteBlocks(pView);

	// close mapping
	UnmapViewOfFile(pView);
	CloseHandle(hDiskMap);
	fileDisk.Close();
	return TRUE;
}


void CControlFile::Unload()
{
	if (m_bLoaded)
	{
		Save();

		if (m_hIPCMap)
		{
			if (m_pViewOfIPC)
			{
				UnmapViewOfFile(m_pViewOfIPC);
			}

			CloseHandle(m_hIPCMap);
		}
		m_bLoaded = false;
	}

    if (m_pLock)
    {
        delete m_pLock;
    }	

}
	
CDownloadGroup* CControlFile::GetNext()
{
	if (m_posNext)
		return m_lsGroups.GetNext(m_posNext);
	else
		return NULL;
}

void CControlFile::Rewind()
{
	m_posNext = m_lsGroups.GetHeadPosition();
}

//////////////////////////////////////////////////////////////////////////////
// Load from memory
//////////////////////////////////////////////////////////////////////////////
void CControlFile::LoadFromIPC()
{
	if ( m_pViewOfIPC &&  m_pLock )
    {
        // get read access to common memory region
        if ( m_pLock->WaitToRead() )
        {
			LoadBlocks(m_pViewOfIPC);            
			m_pLock->Done();
	
        }
    }
}

void CControlFile::LoadBlocks(BYTE* view)
{
	int nBlockCount = *view;
	view++;

	for (int nBlock=0;nBlock<nBlockCount;nBlock++)
	{
		int nBlockType = *view;
		view++;

		WORD nBlockSize = *(WORD *)view;
		view += 2;

		switch (nBlockType)
		{
		case DAV_BLOCK_IPC_GROUP:
			LoadBlockGroup(view, DAV_IPC);
			break;
		case DAV_BLOCK_DISK_GROUP:
			LoadBlockGroup(view, DAV_DISK);
		}
		view += nBlockSize;
	}
}

void CControlFile::LoadBlockGroup(BYTE* view, BOOL bReadGroupData)
{
	CDownloadGroup* group = new CDownloadGroup();
	char buf[1024];
	CString log;
	
	int nFileCount = *view;
	view++;
	
	if (bReadGroupData)
	{
		/*		Byte2				Lngd p namn n
				Byte3..4			Lngd p savePath (WORD) m
				Byte5				Procent nedladdad
				Byte6..9			Bytes nedladdade
				Byte10				Status
				Byte11				Senast aktiv fil
				ByteA..B			Namn
				ByteC..D			SavePath	*/

		int nNameLength = *view;
		view++;

		int nSavePathLength = *(WORD *)view;
		view+=2;

		group->SetDone(*view);
		view++;

		group->SetBytesDownloaded(*(int *)view);
		view+=4;

		group->SetStatus(*view);
		view++;

		group->SetLastActiveFile(*view);
		view++;

		group->SetID(*(int *)view);
		view+=4;

		group->SetName((const char *)view);
		view+=nNameLength;

		group->SetPathname((const char *)view);
		view+=nSavePathLength;
	}

	group->SetSize(nFileCount);
	for (int nFile=0;nFile<nFileCount;nFile++)
	{
		CDownloadQueuedFile* file = new CDownloadQueuedFile();
		
		file->SetId(*(int *)view);
		view+=4;
		
		int nFilenameLength = *view;
		view++;
		int nDirnameLength = *view;
		view++;

		file->SetPing(*(short *)view);
		view+=2;

		file->SetDistance(*view);
		view++;

		file->SetAccess(*view);
		view++;

		file->SetFilesize(*(int *)view);
		view+=4;

		strcpy(buf,(char *)view);
		file->SetIP(buf);
		view+=16;

		strcpy(buf,(char *)view);
		file->SetFilename(buf);
		view+=nFilenameLength;

		strcpy(buf,(char *)view);
		file->SetDirname(buf);
		view+=nDirnameLength;

		log.Format("NEW FILE\r\nFilename:\t%s\r\nShare:\t%s\r\nIP:\t%s\r\nFilesize:\t%d\r\nPing:\t%d\n",
			file->GetFilename(),
			file->GetDirname(),
			file->GetIP(),
			file->GetFilesize(),
			file->GetPing()
			);
		
		TRACE(log);

		group->AddFile(nFile,file);
	}

	m_lsGroups.AddTail(group);
}

//////////////////////////////////////////////////////////////////////////////
// Write to memory (and mapped file)
//////////////////////////////////////////////////////////////////////////////

int CControlFile::PrepareData()
{
	int nSize = 0;
	POSITION pos = m_lsGroups.GetHeadPosition();
	while (pos)
	{
		CDownloadGroup* group = m_lsGroups.GetNext(pos);		
		group->PrepareData();
		nSize+=group->GetDataSize();
	}
	return nSize;
}

void CControlFile::WriteBlocks(BYTE* view)
{
	*view = (BYTE)m_lsGroups.GetCount();
	view++;
	
	POSITION pos = m_lsGroups.GetHeadPosition();
	while (pos)
	{
		CDownloadGroup* group = m_lsGroups.GetNext(pos);		
		
		// Block Type
		*view = DAV_BLOCK_DISK_GROUP;
		view++;
		
		// Block size
		*(WORD *)view = (WORD)group->GetDataSize();
		view+=2;
		
		// Block data
		memcpy(view,group->GetData(),group->GetDataSize());
		view+=group->GetDataSize();
		
		group->FreeData();
	}
}

void CControlFile::DeleteGroup(CDownloadGroup *group)
{
	POSITION pos = m_lsGroups.Find(group);
	if (pos)
	{
		m_lsGroups.RemoveAt(pos);
		delete group;
	}
}
