 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

// DownloadGroup.cpp: implementation of the CDownloadGroup class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Download.h"
#include "DownloadGroup.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDownloadGroup::CDownloadGroup()
{
	m_nDone = 0;
	m_nStatus = DDS_WAITING;
	GenerateStatusString();

	// fix id

	m_nID = 0;

	m_nBytesDownloaded = 0;
	m_nLatestActiveFile = 0;

	strcpy(m_strSavePath,"");
	strcpy(m_strStatus,"");
	strcpy(m_strFilesizes,"");
	strcpy(m_strName,"");
}

CDownloadGroup::~CDownloadGroup()
{
/*	while (!m_lsFiles.IsEmpty())
	{
		delete m_lsFiles.GetHead();
		m_lsFiles.RemoveHead();
	}	*/
	
	int nFileCount = m_arFiles.GetSize();
	for (int i=0;i<nFileCount;i++)
	{
		delete m_arFiles.GetAt(i);
	}

	m_arFiles.RemoveAll();
}

CDownloadGroups::CDownloadGroups()
{
	m_bAutoDelete = TRUE;
}

CDownloadGroups::~CDownloadGroups()
{
	if (m_bAutoDelete)
	{
		while (!IsEmpty())
		{
			delete GetHead();
			RemoveHead();
		}
	}
}

//////////////////////////////////////////////////////////////////////
// Getters
//////////////////////////////////////////////////////////////////////

int CDownloadGroup::GetStatus()
{
	return m_nStatus;
}

int CDownloadGroup::GetBytesDownloaded()
{
	return m_nBytesDownloaded;
}

int CDownloadGroup::GetLastActiveFile()
{
	return m_nLatestActiveFile;
}

LPCSTR CDownloadGroup::GetPathname()
{
	return m_strSavePath;
}

LPCSTR CDownloadGroup::GetName()
{
	return m_strName;
}

CDownloadQueuedFile* CDownloadGroup::GetFile(int nIdx)
{
	if (nIdx > m_arFiles.GetUpperBound())
		return NULL;

	return m_arFiles.GetAt(nIdx);
/*	
	if (m_posNextFile)
		return m_lsFiles.GetNext(m_posNextFile);
	else 
		return NULL;*/
}

int CDownloadGroup::GetShareCount()
{
	/*return m_lsFiles.GetCount();*/
	return m_arFiles.GetSize();
}

LPCSTR CDownloadGroup::GetFirstFilename()
{
	ASSERT(m_arFiles.GetSize() > 0);

	return m_arFiles.GetAt(0)->GetFilename();
}

LPCSTR CDownloadGroup::GetFilesizes()
{
	return m_strFilesizes;
}

LPCSTR CDownloadGroup::GetStatusString()
{
	GenerateStatusString();
	return m_strStatus;
}

int CDownloadGroup::GetDone()
{
	return m_nDone;
}

int CDownloadGroup::GetID()
{
	if (!m_nID)
	{
		GenerateID();
	}
	return m_nID;
}

//////////////////////////////////////////////////////////////////////
// Setters
//////////////////////////////////////////////////////////////////////

void CDownloadGroup::SetStatus(UINT nStatus)
{
	m_nStatus = nStatus;
	GenerateStatusString();
}

void CDownloadGroup::SetPathname(const char* strPathname)
{
	strcpy(m_strSavePath,strPathname);
}

void CDownloadGroup::SetName(const char* strName)
{
	strcpy(m_strName,strName);
}

void CDownloadGroup::SetDone(int nDone)
{
	m_nDone = nDone;
}

void CDownloadGroup::SetID(int nID)
{
	m_nID = nID;
}

void CDownloadGroup::SetBytesDownloaded(int nBytes)
{
	m_nBytesDownloaded = nBytes;
}

void CDownloadGroup::AddBytesDownloaded(int nBytes)
{
	m_nBytesDownloaded += nBytes;
}

void CDownloadGroup::SetLastActiveFile(int nLatest)
{
	m_nLatestActiveFile = nLatest;
}

void CDownloadGroup::SetSize(int nSize)
{
	m_arFiles.SetSize(nSize);
}

//////////////////////////////////////////////////////////////////////
// Modifiers
//////////////////////////////////////////////////////////////////////

void CDownloadGroup::AddFile(int nIdx, CDownloadQueuedFile* file)
{
	m_arFiles.SetAt(nIdx,file);

	if (nIdx == m_arFiles.GetUpperBound())
		GenerateFilesizes();

}

void CDownloadGroup::Sort(UINT nSortFunction)
{
	TRACE("Sorting not done!\n");
}

//////////////////////////////////////////////////////////////////////
// Internal procs
//////////////////////////////////////////////////////////////////////

void CDownloadGroup::GenerateID()
{
	if (!m_nID)
	{
		CWinApp* app = AfxGetApp();
		m_nID = app->GetProfileInt("Group","NextID",1);
		app->WriteProfileInt("Group","NextID",m_nID+1);
		TRACE("Group id: %d\n", m_nID);
	}
}
void CDownloadGroup::GenerateFilesizes()
{
	int nMax = 0;
	int nMin = 8000000; // I doubt that anyone has a bigger file than 8gb 

	int nFileCount = m_arFiles.GetUpperBound() + 1;
	CDownloadQueuedFile *file;
	for (int i=0;i<nFileCount;i++)
	{
		file = m_arFiles.GetAt(i);
		nMax = __max(nMax, file->GetFilesize());
		nMin = __min(nMin, file->GetFilesize());
	}

	if (nMin == nMax)
		sprintf(m_strFilesizes,"%d KB",nMax);
	else
		sprintf(m_strFilesizes,"%d KB - %d KB",nMin,nMax);
}

void CDownloadGroup::GenerateStatusString()
{
	switch(m_nStatus)
	{
	case DDS_WAITING:
		strcpy(m_strStatus,"Waiting");
		break;
	case DDS_DOWNLOADING:
		strcpy(m_strStatus,"Downloading");
		break;
	case DDS_COMPLETE:
		strcpy(m_strStatus,"Complete");
		break;
	case DDS_ABORTED:
		strcpy(m_strStatus,"Aborted");
		break;
	case DDS_OPENING:
		strcpy(m_strStatus,"Opening");
		break;
	}
}

//////////////////////////////////////////////////////////////////////
// Datamaking
//////////////////////////////////////////////////////////////////////
	
void CDownloadGroup::PrepareData()
{
	BYTE* pData;
	CDownloadQueuedFile* file;

	/*
			Byte1				Antal IPC filer k
			Byte2				Lngd p namn n
			Byte3..4			Lngd p savePath (WORD) m
			Byte5				Procent nedladdad
			Byte6..9			Bytes nedladdade
			Byte10				Status
			Byte11				Senast aktiv fil
			Byte12..15			ID (int)
			ByteA..B			Namn
			ByteC..D			SavePath
			k st IPC filer

			*/

	// Find out size och data
	m_nDataSize = 15;
	int nNameLength = strlen(m_strName) + 1;
	int nSavePathLength = strlen(m_strSavePath) + 1;
	m_nDataSize += nNameLength;
	m_nDataSize += nSavePathLength;
	int nFileCount = m_arFiles.GetSize();
	for (int i=0;i<nFileCount;i++)
	{
		file = m_arFiles.GetAt(i);
		file->PrepareData();
		m_nDataSize += file->GetDataSize();
	}

	// Allocate data
	m_pData = new BYTE[m_nDataSize];
	pData = m_pData;

	// write stuff
	*pData = (BYTE)m_arFiles.GetSize();
	pData++;
	
	*pData = nNameLength;
	pData++;

	*(WORD *)pData = (WORD)nSavePathLength;
	pData+=2;

	*pData = (BYTE)m_nDone;
	pData++;

	*(int *)pData = m_nBytesDownloaded;
	pData+=4;

	*pData = (BYTE)m_nStatus;
	pData++;

	*pData = (BYTE)m_nLatestActiveFile;
	pData++;

	*(int *)pData = (int)m_nID;
	pData+=4;

	strcpy((char *)pData,m_strName);
	pData+=nNameLength;

	strcpy((char *)pData,m_strSavePath);
	pData+=nSavePathLength;
	
	for (i=0;i<nFileCount;i++)
	{
		file = m_arFiles.GetAt(i);
		memcpy(pData,file->GetData(),file->GetDataSize());
		pData+=file->GetDataSize();
		file->FreeData();
	}
}

int CDownloadGroup::GetDataSize()
{
	return m_nDataSize;	
}

BYTE* CDownloadGroup::GetData()
{
	return m_pData;
}

void CDownloadGroup::FreeData()
{
	delete[] m_pData;
}
