#if !defined(AFX_DOWNLOADGROUP_H__A8617C3F_C6FF_49A5_9D68_DE723F4C3821__INCLUDED_)
#define AFX_DOWNLOADGROUP_H__A8617C3F_C6FF_49A5_9D68_DE723F4C3821__INCLUDED_

 /*******************************************************************
 This file is part of Davves.

    Davves is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Davves is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Davves; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "DownloadQueuedFile.h"
#include <afxtempl.h>

#define DAV_DOWNLOAD_SORT_PING 1
#define DAV_DOWNLOAD_SORT_DISTANCE 2
#define DAV_DOWNLOAD_SORT_BOTH 3

#define DDS_WAITING 1
#define DDS_DOWNLOADING 2
#define DDS_COMPLETE 3
#define DDS_ABORTED 4
#define DDS_OPENING 5

typedef CTypedPtrList<CPtrList,CDownloadQueuedFile*> CDownloadQueuedFiles;
typedef CTypedPtrArray<CPtrArray,CDownloadQueuedFile*> CFileArray;

class CDownloadGroup  
{
private:
	// For Datamaking
	BYTE* m_pData;
	int m_nDataSize;

	// Info about group
	char m_strSavePath[1024];			// Where to save this download
	char m_strName[256];				// Tag of group
	char m_strFilesizes[100];			// xxxxx kb - xxxxxx kb or xxxxx kb if all same
	char m_strStatus[30];				// pre-generated status string

	POSITION m_posNextFile;				// List locator
	CFileArray m_arFiles;		// List of files in group
	UINT m_nStatus;						// Current Status, One of DDS_*
	int m_nDone;						// value between 1-100, that is done
	int m_nCurrentFile;					// Latest or current download source
	int m_nID;							// Index 
	int m_nBytesDownloaded;
	int m_nLatestActiveFile;

	// Internal procs
	void GenerateFilesizes();
	void GenerateStatusString();
	void GenerateID();
public:
	CDownloadGroup();
	virtual ~CDownloadGroup();
	
	// Getters
	int			GetStatus();					// One of DDS_*
	int			GetShareCount();
	LPCSTR		GetStatusString();
	LPCSTR		GetPathname();
	LPCSTR		GetFirstFilename();
	LPCSTR		GetName();
	CDownloadQueuedFile* GetFile(int nIdx);
	LPCSTR		GetFilesizes();
	int			GetDone();
	int			GetID();
	int			GetBytesDownloaded();
	int			GetLastActiveFile();
	
	// Setters
	void SetID(int nID);
	void SetStatus(UINT nStatus);
	void SetPathname(LPCSTR strPathname);
	void SetName(LPCSTR strName);
	void SetDone(int nDone);
	void SetBytesDownloaded(int nBytes);
	void AddBytesDownloaded(int nBytes);
	void SetLastActiveFile(int nLatest);
	void SetSize(int nSize);

	// Modifiers
	void AddFile(int nIdx, CDownloadQueuedFile* file); // Add file to tail
	void Sort(UINT nSortFunction);

	// Datamaking 
	void PrepareData();
	int GetDataSize();
	BYTE* GetData();
	void FreeData();
};

class CDownloadGroups : public CTypedPtrList<CPtrList,CDownloadGroup*>
{
public:
	CDownloadGroups();
	~CDownloadGroups();
	BOOL m_bAutoDelete;
};


#endif // !defined(AFX_DOWNLOADGROUP_H__A8617C3F_C6FF_49A5_9D68_DE723F4C3821__INCLUDED_)
