// DownloadThread.cpp: implementation of the CDownloadThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "download.h"
#include "DownloadThread.h"
#include "HighTime.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDownloadThread::CDownloadThread(CDownloadGroup *group)
{
	m_pGroup = group;
	m_Status.m_pGroup = m_pGroup;
	m_Status.m_nProcent = m_pGroup->GetDone();

	CDownloadThread();
	m_nCommand = DDT_CONTINUE;
	m_lBytesRead = 0;
}

CDownloadThread::CDownloadThread()
{
	m_lBytesRead = 0;
	m_bResumeAllowed = true;
}

CDownloadThread::~CDownloadThread()
{
}

////////////////////////////////////////////////////////////////////////////
// Setters
////////////////////////////////////////////////////////////////////////////

void CDownloadThread::SetGroup(CDownloadGroup *group)
{
	m_pGroup = group;
	m_Status.m_pGroup = group;
}

////////////////////////////////////////////////////////////////////////////
// Getters
////////////////////////////////////////////////////////////////////////////

CDownloadGroup* CDownloadThread::GetGroup()
{
	return m_pGroup;
}

////////////////////////////////////////////////////////////////////////////
// Thread Proc
////////////////////////////////////////////////////////////////////////////

UINT CDownloadThread::DownloadProc(LPVOID pParam)
{
	CDownloadThread* pObj = (CDownloadThread *)pParam;

	pObj->m_bIsRunning = true;
	int nRet =  pObj->Download();
	pObj->m_bIsRunning = false;
	
	return nRet;
}


////////////////////////////////////////////////////////////////////////////
// Public procs
////////////////////////////////////////////////////////////////////////////

void CDownloadThread::Start()
{
	m_hThread = AfxBeginThread(DownloadProc,this);
}

bool CDownloadThread::IsRunning()
{
	return m_bIsRunning;
}

void CDownloadThread::Stop()
{
	if (IsRunning())
	{
		m_nCommand = DDT_STOP;
		m_hThread->m_bAutoDelete = FALSE;
		WaitForSingleObject(m_hThread->m_hThread,INFINITE);
		delete m_hThread;
	}
}

void CDownloadThread::Abort()
{
	m_nCommand = DDT_ABORT;
}


////////////////////////////////////////////////////////////////////////////
// Internal procs
////////////////////////////////////////////////////////////////////////////

int CDownloadThread::Download()
{
	int nCopyRet, nRemoteRet, nLocalRet;

	m_pGroup->Sort(0);
	nLocalRet = OpenLocalFile();
	
	if (nLocalRet == DAV_DOWNLOAD_LOCAL_FAILED)
	{
		UpdateParent(DDS_ABORTED);
		return 0;
	}

	do
	{
		nRemoteRet = OpenRemoteFile();
	
		if (nLocalRet == DAV_DOWNLOAD_LOCAL_RESUMED &&
			nRemoteRet == DAV_DOWNLOAD_REMOTE_RESUMED)
		{
			SyncFiles();
		}
		else if (nLocalRet == DAV_DOWNLOAD_LOCAL_RESUMED)
		{
			UpdateParent("Wrong filesize found on disk! Can't resume file.");			
			UpdateParent(DDS_ABORTED);	
		}
		else if (nRemoteRet == DAV_DOWNLOAD_REMOTE_SUCCESS)
		{
			nCopyRet = Copy();
		}
	} while (nCopyRet != DAV_DOWNLOAD_COPY_COMPLETE && nRemoteRet != DAV_DOWNLOAD_REMOTE_ENDOFFILES);
 
	CloseFiles();

	return 0;
}

void CDownloadThread::CloseFiles()
{
	if (m_fileLocal.m_hFile != CFile::hFileNull)
		m_fileLocal.Close();

	if (m_fileRemote.m_hFile != CFile::hFileNull)
		m_fileRemote.Close();
}

int CDownloadThread::OpenRemoteFile()
{
	UpdateParent(DDS_OPENING);
	
	// Assume at least one file in group;
	CDownloadQueuedFile* pFile;
	char buf[1024];

	int nIdx = m_pGroup->GetLastActiveFile();
	do
	{
		pFile = m_pGroup->GetFile(nIdx);
		if (pFile)
		{
			strcpy(buf,pFile->GetDirname());
			strcat(buf,pFile->GetFilename());
			UpdateParent("Trying to open file from:");
			UpdateParent(buf);

			if (m_fileRemote.Open(buf,CFile::modeRead,NULL))
			{
				break;
			}
		}
		nIdx++;
	} while (pFile);
			
	if (m_fileRemote.m_hFile == CFile::hFileNull)
	{
		UpdateParent("Failed to open file!");
		UpdateParent(DDS_ABORTED);
		return DAV_DOWNLOAD_REMOTE_ENDOFFILES;
	}
	else
	{
		UpdateParent("File opened successfully!");
		m_fileRemote.GetStatus(m_statusRemote);

		// Check if new file is same size. If not, resume is fucked.
		if (m_statusRemote.m_size == m_pGroup->GetFile(m_pGroup->GetLastActiveFile())->GetFilesize())
		{
			m_pGroup->SetLastActiveFile(nIdx);
			return DAV_DOWNLOAD_REMOTE_RESUMED;
		}
		else
		{
			m_pGroup->SetLastActiveFile(nIdx);
			return DAV_DOWNLOAD_REMOTE_SUCCESS;
		}
	}
}	

int CDownloadThread::OpenLocalFile()
{
	if (m_pGroup->GetStatus() == DDS_COMPLETE)
	{
		UpdateParent("File already complete!");
		return DAV_DOWNLOAD_LOCAL_FAILED;
	}


	UpdateParent(DDS_OPENING);
	// Check if file exists
	if (CFile::GetStatus((LPSTR)m_pGroup->GetPathname(),m_statusLocal))
	{
		// File exists
		if (m_statusLocal.m_size == m_pGroup->GetBytesDownloaded())
		{
			// It's the right one
			UpdateParent("Found unfinished file on disk. Resuming...");
			if (m_fileLocal.Open((LPSTR)m_pGroup->GetPathname(),
								CFile::modeWrite | 
								CFile::modeNoTruncate | 
								CFile::modeCreate | 
								CFile::shareDenyWrite,
								NULL))
				return DAV_DOWNLOAD_LOCAL_RESUMED;
			else
				return DAV_DOWNLOAD_LOCAL_FAILED;
		}
		else
		{
			// This is not rigth.. What to do?
			// Check if i'm allowed to ask about overwriting
			int nRet;
			UpdateParent("Found file on disk, but with wrong size!");
			nRet = AfxMessageBox("Found file on disk! Do you want to overwrite this file?",MB_YESNO);
			if ((nRet == IDYES) && IsMatching() && m_fileLocal.Open((LPSTR)m_pGroup->GetPathname(),
													CFile::modeWrite | 
													CFile::modeCreate | 
													CFile::shareDenyWrite,
													NULL)) 
			{
				m_pGroup->SetBytesDownloaded(0);
				return DAV_DOWNLOAD_LOCAL_SUCCESS;
			}
			else
				return DAV_DOWNLOAD_LOCAL_FAILED;

		}
	}
	else if (m_fileLocal.Open((LPSTR)m_pGroup->GetPathname(),
							CFile::modeWrite | 
							CFile::modeCreate |
							CFile::shareDenyWrite,
							NULL)) 
	{
		m_fileLocal.GetStatus(m_statusLocal);
		UpdateParent("Opened local file.");
		return DAV_DOWNLOAD_LOCAL_SUCCESS;
	}
	else
		return DAV_DOWNLOAD_LOCAL_FAILED;
	
}

int CDownloadThread::Copy()
{
	// Assume both files is open
	// Set up buffer
	DWORD dwRead;
	BYTE buffer[DAV_DOWNLOAD_BUFFER_SIZE];

	// Setup status
	UpdateParent(DDS_DOWNLOADING);
	UpdateParent("Starting to copy ... ");

	// vars for timekeeping
	CHighTime startTime, nowTime, lastSecondTime;
	CHighTimeSpan downloadSpan, secondSpan;
	startTime = CHighTime::GetPresentTime();
	lastSecondTime = CHighTime::GetPresentTime();
	LONG fetchedThisSec=0;
	
	TRY
	{
		do
		{
			// Do some I/O
			dwRead = m_fileRemote.Read(buffer, DAV_DOWNLOAD_BUFFER_SIZE);
			m_fileLocal.Write(buffer, dwRead);

			m_lBytesRead += dwRead;
			fetchedThisSec += dwRead;

			// Update status			
			nowTime = CHighTime::GetPresentTime();
			secondSpan = nowTime - lastSecondTime;
			if ((secondSpan.GetTotalMilliSeconds() > 1000) || !dwRead)
			{
				downloadSpan = nowTime - lastSecondTime;
				lastSecondTime = CHighTime::GetPresentTime();

				UpdateParent(
					(int)(((double)(m_lBytesRead) / (double)m_statusRemote.m_size) * 100),		// percentage
					m_lBytesRead,																// bytes read	
					(double)(fetchedThisSec / downloadSpan.GetTotalMilliSeconds())*(1024/1000)	// speed in KB/sec
					);

				fetchedThisSec = 0;
			}
		}
		while (dwRead>0 && m_nCommand==DDT_CONTINUE);
	}
	CATCH(CFileException, pEx)
	{
		TRACE("Catched file exception, %d\n",pEx->m_lOsError);
		UpdateParent("Error while downloading... ");
		m_pGroup->SetBytesDownloaded(m_lBytesRead);
		return DDS_ABORTED;
	}
	END_CATCH

	m_pGroup->SetBytesDownloaded(m_lBytesRead);

	if (m_nCommand == DDT_ABORT)
	{
		UpdateParent("Download aborted by user.");
		UpdateParent(DDS_ABORTED);
	}
	else if (m_nCommand == DDT_CONTINUE)
	{
		UpdateParent("Copy completed successfully!");
		UpdateParent(DDS_COMPLETE);
	}
	else if (m_nCommand == DDT_STOP)
	{
		// send nothing.. main thread is waiting for us to terminate
	}

	return DAV_DOWNLOAD_COPY_COMPLETE;
}

void CDownloadThread::SetId(int nId)
{
	m_nId = nId;
	m_Status.m_nID = nId;
}

void CDownloadThread::UpdateParent()
{
	ASSERT(m_nCommand != DDT_STOP);
	SendMessage(m_pParentWnd,WMU_DOWNLOAD,DAV_DOWNLOAD_UPDATE,(long)&m_Status);
}

void CDownloadThread::UpdateParent(int nNewStatus)
{
	m_Status.m_nType = DGS_STATUSCHANGE;
	m_Status.m_nStatus = nNewStatus;
	UpdateParent();
}

void CDownloadThread::UpdateParent(LPCSTR strMessage)
{
	m_Status.m_nType = DGS_MESSAGE;
	m_Status.SetMessage(strMessage);
	TRACE("%s\n",strMessage);
	UpdateParent();
}

void CDownloadThread::UpdateParent(int nProcent, int nBytesDownloaded, double dSpeed)
{
	m_Status.m_nType = DGS_DOWNLOADUPDATE;
	m_Status.m_nProcent = nProcent;
	m_Status.m_nBytesDownloaded = nBytesDownloaded;
	m_Status.m_dSpeed = dSpeed;
	UpdateParent();
}

bool CDownloadThread::IsMatching()
{
	return true;
}

void CDownloadThread::SyncFiles()
{
	// Assumes both files open and that status is fetched
	m_lBytesRead = m_statusLocal.m_size;

	m_fileRemote.Seek(m_lBytesRead,CFile::begin);
	m_fileLocal.Seek(m_lBytesRead,CFile::begin);
	
}
