// DownloadThread.h: interface for the CDownloadThread class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DOWNLOADTHREAD_H__A1070DCA_A66C_49D9_95AF_4BF3DDB90005__INCLUDED_)
#define AFX_DOWNLOADTHREAD_H__A1070DCA_A66C_49D9_95AF_4BF3DDB90005__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "GroupStatus.h"
#include "DownloadGroup.h"

#define DAV_DOWNLOAD_SOURCE_FAILED 111
#define DAV_DOWNLOAD_LOCAL_FAILED 112
#define DAV_DOWNLOAD_COPY_FAILED 113
#define DAV_DOWNLOAD_LOCAL_RESUMED 114
#define DAV_DOWNLOAD_REMOTE_RESUMED 115
#define DAV_DOWNLOAD_REMOTE_ENDOFFILES 116
#define DAV_DOWNLOAD_COPY_COMPLETE 117
#define DAV_DOWNLOAD_REMOTE_SUCCESS 118
#define DAV_DOWNLOAD_LOCAL_SUCCESS 119

#define DAV_DOWNLOAD_BUFFER_SIZE 16384
#define DAV_DOWNLOAD_UPDATE 1

#define DDT_CONTINUE 0
#define DDT_ABORT 1
#define DDT_STOP 2

class CDownloadThread  
{
private:
	long m_lBytesRead;
	volatile UINT m_nCommand;

	// Attributes
	CDownloadGroup* m_pGroup;
	CFile m_fileRemote;
	CFile m_fileLocal;
	CFileStatus m_statusRemote;
	CFileStatus m_statusLocal;
	
	// Status
	bool m_bIsRunning;

	// Internal procs
	int Download();
	int OpenRemoteFile();
	int OpenLocalFile();
	int Copy();
	void CloseFiles();
	void UpdateParent();
	void UpdateParent(int nNewStatus);
	void UpdateParent(int nProcent, int nBytesDownloaded, double dSpeed);
	void UpdateParent(LPCSTR strMessage);


public:
	CGroupStatus m_Status;
	bool m_bResumeAllowed;
	void SyncFiles();
	bool IsMatching();
	// (De)Construction
	CDownloadThread(CDownloadGroup* group);
	CDownloadThread();
	~CDownloadThread();

	// Setters
	void SetGroup(CDownloadGroup* group);
	void SetId(int nId);

	// Getters
	CDownloadGroup* GetGroup();

	// Thread stuff
	static UINT DownloadProc(LPVOID pParam);
	CWinThread* m_hThread;

	int m_nId; // Used to keep track of downloadthreads

	HWND m_pParentWnd; // Where to send updates

	// Public Procs
	void Start();
	bool IsRunning();
	void Stop();		// Stops thread and waits for it to terminate
	void Abort();		// Just tells the thread to stop as soon it can. 

};

#endif // !defined(AFX_DOWNLOADTHREAD_H__A1070DCA_A66C_49D9_95AF_4BF3DDB90005__INCLUDED_)
