// GroupEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "download.h"
#include "GroupEditDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGroupEditDlg dialog


CGroupEditDlg::CGroupEditDlg(CWnd* pParent /*=NULL*/)
	: CResizableDialog(CGroupEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CGroupEditDlg)
	m_strGroupName = _T("");
	m_strSavePath = _T("");
	//}}AFX_DATA_INIT
}


void CGroupEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CResizableDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGroupEditDlg)
	DDX_Control(pDX, IDC_FILES_LIST, m_ctrlFiles);
	DDX_Text(pDX, IDC_GROUP_NAME_EDIT, m_strGroupName);
	DDX_Text(pDX, IDC_SAVE_PATH_EDIT, m_strSavePath);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGroupEditDlg, CResizableDialog)
	//{{AFX_MSG_MAP(CGroupEditDlg)
	ON_COMMAND(ID_GROUPEDIT_SETASFIRSTTRY, OnGroupeditSetasfirsttry)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY ( NM_CUSTOMDRAW, IDC_FILES_LIST, OnCustomdrawFileList )
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGroupEditDlg message handlers

BOOL CGroupEditDlg::OnInitDialog() 
{
	CResizableDialog::OnInitDialog();

	AddAnchor(IDC_GROUP_NAME_EDIT,TOP_LEFT,TOP_RIGHT);
	AddAnchor(IDC_FILES_LIST,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(IDOK,BOTTOM_RIGHT);	
	AddAnchor(IDCANCEL,BOTTOM_RIGHT);	

	m_ctrlFiles.SetExtendedStyle(LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES| LVS_EX_HEADERDRAGDROP);
	
	m_ctrlFiles.InsertColumn(0,"Directory", LVCFMT_LEFT,200);
	m_ctrlFiles.InsertColumn(1,"Filesize", LVCFMT_LEFT,60);
	m_ctrlFiles.InsertColumn(2,"Filename", LVCFMT_LEFT,150);
	
	UpdateList();
	UpdateData();

	CRect rectDesktop;
	CRect rectMain;

	CWnd *pDesktopWnd = GetDesktopWindow();
	pDesktopWnd->GetWindowRect(rectDesktop);
	GetWindowRect(rectMain);
	SetWindowPos(&wndTop, 
		(rectDesktop.right / 2) - (rectMain.Width() / 2) ,
		(rectDesktop.bottom / 2) - (rectMain.Height() / 2),
		0,0,SWP_NOSIZE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CGroupEditDlg::SetGroup(CDownloadGroup *group)
{
	ASSERT(group);
	m_pGroup = group;

	m_strGroupName = m_pGroup->GetName();
	m_strSavePath = m_pGroup->GetPathname();
}

void CGroupEditDlg::UpdateList()
{
	CString tmp;
	m_ctrlFiles.DeleteAllItems();
	
	int nFileCount = m_pGroup->GetShareCount();
	for (int i=0;i<nFileCount;i++)
	{
		CDownloadQueuedFile* file = m_pGroup->GetFile(i);
		int nIdx = m_ctrlFiles.InsertItem(i,file->GetDirname());
		
		tmp.Format("%d KB",file->GetFilesize());
		m_ctrlFiles.SetItemText(nIdx,1,tmp);

		m_ctrlFiles.SetItemText(nIdx,2,file->GetFilename());

		if (i == m_pGroup->GetLastActiveFile())
		{
			m_ctrlFiles.SetHotItem(nIdx);
		}

		m_ctrlFiles.SetItemData(nIdx,(LPARAM)i);
	}
}

int CGroupEditDlg::FindItem(int nFileID)
{
	LVFINDINFO info;
	info.flags = LVFI_PARAM;
	info.lParam = (LPARAM)nFileID;
	return m_ctrlFiles.FindItem(&info);
}

int CGroupEditDlg::GetSelectedFile() 
{
	POSITION pos = m_ctrlFiles.GetFirstSelectedItemPosition();
	if (pos)
	{
		int nListIdx = m_ctrlFiles.GetNextSelectedItem(pos);
		return (int)m_ctrlFiles.GetItemData(nListIdx);
	}
	else
		return -1;
}

BOOL CGroupEditDlg::UpdateFile(int nFileID) 
{
	CString tmp;
	int nIdx = FindItem(nFileID);
	if (nIdx != -1)
	{
		CDownloadQueuedFile* file = m_pGroup->GetFile(nFileID);
		tmp.Format("%d KB",file->GetFilesize());
		m_ctrlFiles.SetItemText(nIdx,1,tmp);
		m_ctrlFiles.SetItemText(nIdx,2,file->GetFilename());
		m_ctrlFiles.SetItemData(nIdx,(LPARAM)nFileID);
		return TRUE;
	}
	else
		return FALSE;
}


void CGroupEditDlg::OnGroupeditSetasfirsttry() 
{
	int nFileID = GetSelectedFile();
	if (nFileID != -1)
	{
		m_pGroup->SetLastActiveFile(nFileID);
		m_ctrlFiles.Invalidate();
	}
}


void CGroupEditDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CRect rect;
	m_ctrlFiles.GetWindowRect(rect);

	if (rect.PtInRect(point)) // it's for the list ctrl
	{
		CMenu popupMenus;
		if (popupMenus.LoadMenu(IDR_POPUP_MENU))
		{
			POSITION pos = m_ctrlFiles.GetFirstSelectedItemPosition();
			CMenu* pPopup;
			if (pos == NULL)
			{
								//pPopup = popupMenus.GetSubMenu(1);
			}
			else
			{
				pPopup = popupMenus.GetSubMenu(2);
				pPopup->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON,point.x,point.y,this,NULL);	
			}
		}
	}	
	
}

void CGroupEditDlg::OnCustomdrawFileList(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( pNMHDR );

    // Take the default processing unless we set this to something else below.
    *pResult = CDRF_DODEFAULT;

    // First thing - check the draw stage. If it's the control's prepaint
    // stage, then tell Windows we want messages for every item.

    if ( CDDS_PREPAINT == pLVCD->nmcd.dwDrawStage )
        {
        *pResult = CDRF_NOTIFYITEMDRAW;
        }
    else if ( CDDS_ITEMPREPAINT == pLVCD->nmcd.dwDrawStage )
        {
        // This is the prepaint stage for an item. Here's where we set the
        // item's text color. Our return value will tell Windows to draw the
        // item itself, but it will use the new color we set here.
        // We'll cycle the colors through red, green, and light blue.

        COLORREF crText;

		if ( (int)m_ctrlFiles.GetItemData(pLVCD->nmcd.dwItemSpec) == m_pGroup->GetLastActiveFile())
            crText = RGB(255,0,0);
        else
            crText = RGB(0,0,0);

        // Store the color back in the NMLVCUSTOMDRAW struct.
        pLVCD->clrText = crText;

        // Tell Windows to paint the control itself.
        *pResult = CDRF_DODEFAULT;
        }

}
